#!/opt/edge/venv/bin/python3.12

from edge import eventlog
from edge.support import cgi
from edge.webserver.sysconf.api_controller import ControllerApi


class _HandlerFactory:
    """
    GET   /api/about   /
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        """
        :raise CgiError:
        """
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._about__get
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _about__get(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 0:
            eventlog.log_i('fail: http404')
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            dev_info = ControllerApi.about()
            return cgi.CgiResponse(body=dev_info)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-about', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
