#!/opt/edge/venv/bin/python3.12

from typing import List

from edge import eventlog
from edge.support import cgi
from edge.webserver.sysconf.api_root import RootApi


class _HandlerFactory:
    """
    GET   /api/v2/sys/conf/root   /root/info
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._root__get_root_info
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _root__get_root_info(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        if request.path_info[0] != 'root':
            raise cgi.CgiError404

        if request.path_info[1] != 'info':
            raise cgi.CgiError404

        request.assert_body_absent()
        request.assert_body_absent()

        def unpack_unit_refs(query_params: List[str]) -> List[str]:
            uimod_refs = []
            for name_value in query_params:
                name, value = name_value.split('=', 1)
                if name == 'ref':
                    uimod_refs.append(value)

            return uimod_refs

        def handler() -> cgi.CgiResponse:
            unit_refs = unpack_unit_refs(request.query_params)
            units_info = RootApi.get_units_info(unit_refs)
            return cgi.CgiResponse(body={'info': units_info})

        return handler


if __name__ == '__main__':
    eventlog.open_log('root-cgi', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
