from typing import Dict

from edge import eventlog
from edge.image.types.footer import ImFooterColConf, ImFooterRowConf, ImFooterConf
from edge.types.exceptions import JnodeError


def decode__im_footer_col_conf(col_jnode: Dict) -> ImFooterColConf:
    """
    :raise JnodeError:
    """
    if dict != type(col_jnode):
        eventlog.log_e_param('#/')
        raise JnodeError

    # deserialize #/width
    width = col_jnode.get('width')
    if not (int == type(width) and 1 <= width <= 65535):
        eventlog.log_e_param('#/width')
        raise JnodeError

    # deserialize #/align
    raw_align = col_jnode.get('align')
    if not type(raw_align) in [str]:
        eventlog.log_e_param('#/align')
        raise JnodeError

    if raw_align == 'left':
        align = -1
    elif raw_align == 'center':
        align = 0
    elif raw_align == 'right':
        align = +1
    else:
        eventlog.log_e_param('#/align')
        raise JnodeError

    # deserialize #/format
    template = col_jnode.get('format')
    if not (type(raw_align) in [str] and 0 <= len(template) <= 128):
        eventlog.log_e_param('#/format')
        raise JnodeError

    return ImFooterColConf(width, align, template)


def decode__im_footer_row_conf(row_jnode: Dict) -> ImFooterRowConf:
    """
    :raise JnodeError:
    """
    if dict != type(row_jnode):
        eventlog.log_e_param('#/')
        raise JnodeError

    # deserialize #/enabled
    enabled = row_jnode.get('enabled')
    if bool != type(enabled):
        eventlog.log_e_param('#/enabled')
        raise JnodeError

    if enabled:
        cols_jnode = row_jnode.get('cols')
        if list != type(cols_jnode):
            eventlog.log_e_param('#/cols')
            raise JnodeError

        cols = list(map(decode__im_footer_col_conf, cols_jnode))

    else:
        cols = []

    return ImFooterRowConf(enabled, cols)


def decode__im_footer_conf(footer_jnode: Dict) -> ImFooterConf:
    """
    :raise JnodeError:
    """
    # deserialize #/font-height
    font_size = footer_jnode.get('font-height')
    if not (int == type(font_size) and 8 <= font_size <= 72):
        eventlog.log_e_param('#/font-height')
        raise JnodeError

    # deserialize #/rows
    rows_jnode = footer_jnode.get('rows')
    if not (list == type(rows_jnode) and len(rows_jnode) > 0):
        eventlog.log_e_param('#/rows')
        raise JnodeError

    rows = [row for row in map(decode__im_footer_row_conf, rows_jnode) if row.enabled]

    if len(rows) > 0:
        height = int((1.4 * font_size) * len(rows) + 0.4 * font_size)  # 120%
    else:
        # no active rows in footer
        height = 0

    return ImFooterConf(height, font_size, rows)
