from edge.image.types.markings import RefPoint2d, RefPoint3d, LineConf2, LineConf3, MarkingsConf


def decode__markings_conf(jnode: dict) -> MarkingsConf:
    """
    :param jnode: main.conf#/Renderer/profile/annotation/markings
    """
    lines2d_f = []
    lines3d_f = []

    color_map = {
        'red': (255, 0, 0),
        'green': (0, 255, 0),
        'blue': (0, 0, 255),
        'yellow': (255, 255, 0),
        'magenta': (255, 0, 255),
        'cyan': (0, 255, 255)
    }

    for line_f_jnode in jnode.get('line-f', []):
        active = line_f_jnode.get('active', True)
        if not active:
            continue

        color = line_f_jnode.get('color')
        rgb = color_map.get(color, (0, 255, 0))

        point3 = line_f_jnode.get('point3')
        point2 = line_f_jnode.get('point2')
        if point2 is not None:
            x0, x1, y0, y1 = point2
            dist3 = line_f_jnode['dist3']
            lines2d_f.append(LineConf2(RefPoint2d(x0 / x1, y0 / y1), dist3, rgb))
        elif point3 is not None:
            f, l, u = point3
            dist3 = line_f_jnode['dist3']
            lines3d_f.append(LineConf3(RefPoint3d(f, l, u), dist3, rgb))

    lines2d_l = []
    lines3d_l = []

    for line_l_jnode in jnode.get('line-l', []):
        active = line_l_jnode.get('active', True)
        if not active:
            continue

        color = line_l_jnode.get('color')
        rgb = color_map.get(color, (255, 255, 0))

        point3 = line_l_jnode.get('point3')
        point2 = line_l_jnode.get('point2')
        if point2 is not None:
            x0, x1, y0, y1 = point2
            dist3 = line_l_jnode['dist3']
            lines2d_l.append(LineConf2(RefPoint2d(x0 / x1, y0 / y1), dist3, rgb))
        elif point3 is not None:
            f, l, u = point3
            dist3 = line_l_jnode['dist3']
            lines3d_l.append(LineConf3(RefPoint3d(f, l, u), dist3, rgb))

    return MarkingsConf(lines2d_f, lines2d_l, lines3d_f, lines3d_l, jnode.get('vehicle', False))
