from abc import abstractmethod
from typing import Final

from edge.services.types.service import IService
from edge.types.geoposition import Geoposition


class SkyValidatorConf:
    def __init__(self, timeout_sec: int, max_pdop: float) -> None:
        self.timeout_usec: Final[int] = int(timeout_sec * 1e6)
        self.max_pdop: Final[format] = max_pdop


class ChangelogConf:
    def __init__(self, max_depth_records: int, max_depth_sec: int, max_lookup_dist_sec: int) -> None:
        self.max_depth_records: Final[int] = max_depth_records
        self.max_depth_usec: Final[int] = max_depth_sec * 1000000
        self.max_lookup_dist_usec: Final[int] = max_lookup_dist_sec * 1000000


class GnssServiceConf:
    PROVIDER_STATIC = 'static'
    PROVIDER_GPSD = 'gpsd'

    def __init__(self, provider: str,
                 changelog: ChangelogConf,
                 geoposition: Geoposition | None = None,
                 endpoint: tuple[str, int] | None = None,
                 azimuth: float | None = None,
                 no_gpst_text: str = '',
                 max_pdop: float | None = None):

        self.provider = provider
        self.azimuth = azimuth
        self.no_gpst_text = no_gpst_text

        self.static_geoposition = geoposition

        self.gpsd_endpoint = endpoint
        self.gpsd_changelog: ChangelogConf = changelog
        self.gpsd_sky_validator: SkyValidatorConf | None = None if max_pdop is None else SkyValidatorConf(14, max_pdop)


class IGnssService(IService):
    @abstractmethod
    def get_geoposition(self, ts: int) -> Geoposition | None:
        pass
