from edge.services.types.cam import CamServiceConf, SpeedcamConf
from edge.support import fs


def decode__cam_intrinsics_active(intrinsics_path: str) -> bool:
    """
    : camera_path:
        /etc/edge/setupstore/admin/actions/pipeline/0/reprojections/0.json

    :raises JnodeError:
    """
    jnode: dict | None = fs.load_json3(intrinsics_path, default=None)
    if jnode is None:
        return False

    rw_jnode: dict = jnode.get('rw') or dict()
    rw_conf_jnode: dict = rw_jnode.get('conf') or dict()

    return rw_conf_jnode.get('active') or False


def decode__cam_service_conf(jnode: dict | None) -> CamServiceConf | None:
    """
    : camera_path:
        main.conf#/services/cam
    :raises JnodeError:
    """
    if jnode is None:
        return None

    # /etc/edge/setupstore/admin/actions/pipeline/0/reprojections/0.json
    repro_path: str | None = jnode.get('reprojections')
    if repro_path is None:
        return None
    elif not decode__cam_intrinsics_active(repro_path):
        return None

    avg_plate_u: float | None = jnode.get('avg-plate-u')
    with_log: bool = jnode.get('log') or False

    speedcam: SpeedcamConf = SpeedcamConf(avg_plate_u=avg_plate_u, with_log=with_log)
    return CamServiceConf(repro_path, speedcam)
