import pytz

from typing import Dict

from edge import eventlog
from edge.support import fs
from edge.types.exceptions import JnodeError
from edge.services.types.tz import TzServiceConf
from edge.webserver.sysconf.uiconf.time import TimeUiconf


def decode__tz_service_conf(jnode: Dict) -> TzServiceConf:
    """
    :param jnode:
        /etc/edge/setupstore/admin/actions/time.json#/
    :raise JnodeError:
    """
    # deserialize #/rw
    rw_jnode = jnode.get('rw')
    if dict != type(rw_jnode):
        eventlog.log_e_param('#/rw')
        raise JnodeError

    # deserialize #/rw/tz
    tz_name = rw_jnode.get('args', {}).get('tz')
    if not (type(tz_name) in [str] and 3 <= len(tz_name) <= 32):
        eventlog.log_e_param('#/rw/tz')
        raise JnodeError

    if tz_name not in pytz.all_timezones:
        eventlog.log_e_param('#/rw/tz')
        raise JnodeError

    tz_info = pytz.timezone(tz_name)
    return TzServiceConf(tz_info)


def decode__tz_service_conf_file(conf_path: str = TimeUiconf.CONF_PATH) -> TzServiceConf:
    """
    :param conf_path:
        /etc/edge/setupstore/admin/actions/time.json
    :raise JnodeError:
    """
    conf_jnode = fs.load_json3(conf_path, default=None)
    if not conf_jnode:
        eventlog.log_e_param('#/')
        raise JnodeError

    return decode__tz_service_conf(conf_jnode)
