from abc import abstractmethod
from collections.abc import Iterator
from typing import Final

from edge.services.types.service import IService
from edge.support.mask import MaskRegion
from edge.types.shape.point import PointI, PointF
from edge.types.track import LaneTrackItem

type LaneConf = tuple[str, MaskRegion]


class LaneServiceConf:
    def __init__(self, lanes: list[LaneConf], mask_w: int, mask_h: int):
        if mask_h <= 0 or mask_w <= 0:
            if len(lanes) > 0:
                raise ValueError

        self.lanes: Final[list[LaneConf]] = lanes
        self.mask_w: Final[int] = mask_w
        self.mask_h: Final[int] = mask_h


class ILaneService(IService):
    @abstractmethod
    def map_point_abs(self, abs_point: PointI) -> set[str]:
        """
        :param abs_point:
            Visible coordinates are in range [0, mask_w) and [0, mask_h), px
        """
        pass

    @abstractmethod
    def map_point_rel(self, rel_point: PointF) -> set[str]:
        """
        :param rel_point:
            Visible coordinates are in range [0.0, 1.0) and [0.0, 1.0)
        """
        pass

    @abstractmethod
    def map_points_abs(self, point_provider: Iterator[tuple[int, PointI]]) -> tuple[list[LaneTrackItem], set[str]]:
        pass

    @abstractmethod
    def map_points_rel(self, point_provider: Iterator[tuple[int, PointF]]) -> tuple[list[LaneTrackItem], set[str]]:
        pass
