from abc import abstractmethod
from enum import Enum
from typing import Final, List

from edge import eventlog
from edge.services.types.service import IService
from edge.types.timespan import TimeSpan


class TlServiceConf:
    def __init__(self, endpoint: str, tl_ident: int, red_only: bool = False, tl_pressision_ms: int = 0):
        self.endpoint = endpoint
        self.tl_ident = tl_ident
        self.red_only = red_only
        self.accuracy_usec = tl_pressision_ms * 1000


class TlState:
    def __init__(self,
                 r_active: bool = False,
                 a_active: bool = False,
                 g_active: bool = False) -> None:

        self.r_active: Final[bool] = r_active
        self.a_active: Final[bool] = a_active
        self.g_active: Final[bool] = g_active


class TlRoute(Enum):
    TURN_N = 0
    TURN_L = 1
    TURN_R = 2
    TURN_U = 3

    def __str__(self) -> str:
        if self == TlRoute.TURN_N:
            return "STRAIGHT"
        elif self == TlRoute.TURN_L:
            return "L-turn"
        elif self == TlRoute.TURN_R:
            return "R-turn"
        elif self == TlRoute.TURN_U:
            return "U-turn"
        else:
            eventlog.log_e(f'fail: unsupported (route:{self}')
            raise NotImplementedError()


class TlHaltSpanInfo:
    def __init__(self,
                 halt_time: int | None = None,
                 green_amber_time: int | None = None,
                 red_time: int | None = None,
                 red_amber_time: int | None = None) -> None:

        self.halt_time: Final[int | None] = halt_time
        self.green_amber_time: Final[int | None] = green_amber_time
        self.red_amber_time: Final[int | None] = red_amber_time
        self.red_time: Final[int | None] = red_time


class ITlService(IService):
    @abstractmethod
    def get_state(self, ts: int) -> TlState:
        pass

    @abstractmethod
    def get_halt_spans(self, min_ts: int, max_ts: int, route: TlRoute | None = None) -> List[TimeSpan]:
        pass

    @abstractmethod
    def get_halt_span_info(self, halt_span_ts: int, route: TlRoute | None = None) -> TlHaltSpanInfo:
        pass
