from datetime import time, datetime
from typing import Dict, Final, List, Optional, Tuple

from edge.services.types.tz import ITzService


class ScheduleConf:
    def __init__(self, min_time: time, max_time: time, days: List[bool]):
        self.min_time: Final[time] = min_time
        self.max_time: Final[time] = max_time
        self.days_to_bool: Final[List[bool]] = days


class Schedule:
    def __init__(self, conf: Optional[ScheduleConf], tz_service: ITzService):
        self._disabled = conf is None
        if self._disabled:
            return

        self._tz_service = tz_service
        self._day_to_limits_map: Dict[int, List[Tuple[time, time]] | bool] = {
            0: False,
            1: False,
            2: False,
            3: False,
            4: False,
            5: False,
            6: False
        }

        time00 = time(hour=0, minute=0, second=0, microsecond=0)

        if conf.min_time == conf.max_time == time00:
            for day_index in range(0, len(conf.days_to_bool)):
                day_limits = conf.days_to_bool[day_index]
                self._day_to_limits_map[day_index] = day_limits

        elif conf.min_time < conf.max_time:
            for day_index in range(len(conf.days_to_bool)):
                if conf.days_to_bool[day_index]:
                    self._day_to_limits_map[day_index] = [(conf.min_time, conf.max_time)]
                else:
                    self._day_to_limits_map[day_index] = False

        elif conf.min_time > conf.max_time:
            time24 = time(hour=23, minute=59, second=59, microsecond=9999)
            for day_index in range(len(conf.days_to_bool)):
                day_limits = conf.days_to_bool[day_index]
                next_day_index = (day_index + 1) % 7
                if day_limits:
                    if type(self._day_to_limits_map[day_index]) == list:
                        self._day_to_limits_map[day_index].append((conf.min_time, time24))
                    else:
                        self._day_to_limits_map[day_index] = [(conf.min_time, time24)]

                    self._day_to_limits_map[next_day_index] = [(time00, conf.max_time)]

                else:
                    self._day_to_limits_map[day_index] = False

    def check_ts_usec(self, ts_usec: int) -> bool:
        if self._disabled:
            return True

        dt_local: datetime = self._tz_service.map_ts_usec_to_datetime_local(ts_usec)

        day_time_ranges: List[Tuple[time, time]] | bool = self._day_to_limits_map.get(dt_local.weekday())

        if type(day_time_ranges) == bool:
            return day_time_ranges

        elif type(day_time_ranges) == list:
            for day_time_range in day_time_ranges:
                if day_time_range[0] < dt_local.time() < day_time_range[1]:
                    return True

        return False
