from edge.types.imageref import ImageRef


def encode__image_ref(self: ImageRef) -> dict:
    return {
        'Ts': self.ts,
        'Url': self.url,
        'Codec': self.codec,
        'Wh': [self.width, self.height]
    }


def decode__image_ref(jnode: dict) -> ImageRef:
    blob_ts = jnode['Ts']
    blob_url = jnode.get('Url')
    if blob_url is None or len(blob_url) == 0:
        blob_url = None

    blob_format = jnode['Codec']
    blob_w, blob_h = jnode['Wh']

    return ImageRef(int(blob_ts), blob_url, blob_format, int(blob_w), int(blob_h))


def decode__image_ref_mg(jnode: dict) -> ImageRef:
    """
    :param jnode: MgScene#/
    """
    blob_ts = jnode['Env']['ExpStartTs']
    ref_jnode = jnode['Frame']
    blob_url = ref_jnode['Url']
    blob_format = ref_jnode['Codec']
    blob_w = ref_jnode['W']
    blob_h = ref_jnode['H']

    return ImageRef(int(blob_ts), blob_url, blob_format, blob_w, blob_h)


def decode__image_ref_mc(jnode: dict) -> ImageRef:
    """
    :param jnode: McScene#/
    :raise ValueError:
    """
    stat_done = 1
    # stat_fail = 2
    if jnode['Status'] != stat_done:
        raise ValueError('Failed to deserialize mc_scene')

    blob_ts = jnode['Ts']
    blob_url = jnode['Url']
    blob_format = jnode['Codec']
    blob_w, blob_h = jnode['SizeWh']

    return ImageRef(int(blob_ts), blob_url, blob_format, blob_w, blob_h)
