from typing import Dict

from edge.types.dir import Dir
from edge.types.shape.quad import QuadI
from edge.types.jnode.shape.quad import decode__quadi
from edge.types.jnode.shape.rect import decode__recti
from edge.types.track import LrTrackItem, DirTrackItem, LaneTrackItem


def decode__lr_track_item(jnode: Dict) -> LrTrackItem:
    """
    :param jnode: TrTrack#/Lr/PlateTrace/:index
    """
    ts = jnode['Ts']
    weight = jnode['W']
    quad: QuadI = decode__quadi(jnode['Q'])

    vehicle_rect = None
    vehicle_rect_jnode = jnode.get('VR')
    if vehicle_rect_jnode:
        vehicle_rect = decode__recti(vehicle_rect_jnode)

    return LrTrackItem(ts, weight, quad, vehicle_rect)


def encode__dir_track_item(self: DirTrackItem) -> Dict:
    """
    :return: TrTrack#/Dir/:index
    """
    return {
        'Ts': self.ts,
        'D': self.dir.value,
    }


def decode__dir_track_item(jnode: Dict) -> DirTrackItem:
    """
    :param jnode: TrTrack#/Dir/:index
    """
    item_ts = jnode['Ts']
    item_dir = Dir(jnode['D'])

    return DirTrackItem(item_ts, item_dir)


def encode__lane_track_item(self: LaneTrackItem) -> Dict:
    """
    :return: TrTrack#/Lane/:index
    """
    return {
        'Ts': self.ts,
        'L': self.lane,
    }


def decode__lane_track_item(jnode: Dict) -> LaneTrackItem:
    """
    :param jnode: TrTrack#/Lane/:index
    """
    item_ts = jnode['Ts']
    item_lane = jnode['L']

    return LaneTrackItem(item_ts, item_lane)
