from typing import Dict, Optional, List

from edge.types.jnode.trackset.dir import decode__dir_track
from edge.types.jnode.trackset.lane import decode__lane_track
from edge.types.jnode.trackset.le import decode__le_track
from edge.types.jnode.trackset.lr import decode__lr_track
from edge.types.jnode.trackset.ra import decode__ra_track
from edge.types.jnode.trackset.spotspeed import decode__spotspeeds
from edge.types.jnode.trackset.wa import decode__wa_track
from edge.types.track import LaneTrack, DirTrack
from edge.types.trackset import Trackset, LrTrack, RaTrack, WaTrack, TrTrack


def decode__trackset(jnode: Dict) -> Optional[Trackset]:
    time_sync = jnode['TimeSync']

    lr_track_jnode = jnode.get('Lr')
    lr_alarms_jnode = jnode.get('Alarms')
    lr_track: Optional[LrTrack] = lr_track_jnode and decode__lr_track(lr_track_jnode, lr_alarms_jnode)

    ra_track_jnode = jnode.get('Ra')
    ra_track: Optional[RaTrack] = ra_track_jnode and decode__ra_track(ra_track_jnode)

    wa_track_jnode = jnode.get('Wa')
    wa_track: Optional[WaTrack] = wa_track_jnode and decode__wa_track(wa_track_jnode)

    lane_jnode = jnode.get('Lane')
    lane_track: Optional[LaneTrack] = lane_jnode and decode__lane_track(lane_jnode)

    dir_jnode = jnode.get('Dir')
    dir_track: Optional[DirTrack] = dir_jnode and decode__dir_track(dir_jnode)

    spot_speeds_jnode = jnode.get('Fusion', {}).get('spotspeeds')
    spot_speeds = spot_speeds_jnode and decode__spotspeeds(spot_speeds_jnode)

    if any((lr_track, ra_track, wa_track, dir_track, lane_track)):
        tr_track = TrTrack.create(lr_track, ra_track, dir_track, lane_track)
        return Trackset(time_sync,
                        jnode=jnode,
                        tr_track=tr_track,
                        lr_track=lr_track,
                        ra_track=ra_track,
                        wa_track=wa_track,
                        spotspeeds=spot_speeds)


def decode__tracksets_le(jnode: Dict) -> List[Trackset]:
    le_tracks_jnode: List[Dict] | None = jnode.get('leddar')
    le_tracks = [decode__le_track(x) for x in le_tracks_jnode]
    # TODO: time_sync ?
    return [Trackset(True, jnode=jnode, le_track=x) for x in le_tracks if x]
