from typing import Final
from uuid import uuid4

from edge.types.dir import Dir
from edge.types.trackset.le import LeTrack
from edge.types.trackset.lr import LrTrack
from edge.types.trackset.ra import RaTrack
from edge.types.trackset.spotspeed import SpotSpeeds
from edge.types.trackset.tr import TrTrack
from edge.types.trackset.wa import WaTrack


class Trackset:
    """
    An immutable set of tracks that are related to the same vehicle.
    It should always be possible to create the BaseEvent from the Trackset
    """
    def __init__(self,
                 time_sync: bool,
                 jnode: dict | None = None,
                 tr_track: TrTrack | None = None,
                 lr_track: LrTrack | None = None,
                 ra_track: RaTrack | None = None,
                 le_track: LeTrack | None = None,
                 wa_track: WaTrack | None = None,
                 spotspeeds: SpotSpeeds | None = None) -> None:

        self.uuid: Final[str] = str(uuid4())
        self.time_sync: Final[bool] = time_sync

        self.jnode: Final[dict | None] = jnode
        self.tr_track: Final[TrTrack | None] = tr_track
        self.lr_track: Final[LrTrack | None] = lr_track
        self.ra_track: Final[RaTrack | None] = ra_track
        self.le_track: Final[LeTrack | None] = le_track
        self.wa_track: Final[WaTrack | None] = wa_track
        self.spotspeeds: Final[SpotSpeeds | None] = spotspeeds

    def get_direction(self) -> Dir:
        return Dir.CODE_U if self.tr_track is None else self.tr_track.dir
