from typing import Final

from edge.types.dir import Dir
from edge.types.shape.quad import QuadI
from edge.types.shape.rect import RectF, RectI


class LeTrackItem:
    def __init__(self, min_ts: int, max_ts: int) -> None:
        self.min_ts: Final[int] = min_ts
        self.max_ts: Final[int] = max_ts

    @property
    def ts(self) -> int:
        return self.min_ts

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def get_bbox_rect_rel(self) -> RectF:
        size = 0.05
        return RectF(0.5 - size, 0.5 - size, 0.5 + size, 0.5 + size)

    # noinspection PyMethodMayBeStatic
    def get_bbox_quad_abs(self, image_w: int, image_h: int) -> QuadI:
        abs_cx = 0.5 * image_w
        abs_fy = 0.5 * image_h
        padding = 0.05 * image_w

        quad_bbox = RectI(
            abs_cx - padding,
            abs_fy - padding,
            abs_cx + padding,
            abs_fy + padding)

        return QuadI.create(quad_bbox)


class LeTrack(list):
    """
    Contains the history of the presence of the object in a specified area (leddar loop)
        - More than one area (leddar loop) can be specified in the sensor configuration
        - More than one LeTrack can be present in the track
    """
    def __init__(self, trace: list[LeTrackItem], ref: str) -> None:
        super().__init__(trace)
        self.ref = ref

    @property
    def min_ts(self) -> int:
        return self[0].ts

    @property
    def max_ts(self) -> int:
        return self[-1].ts

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def get_item_direction(self, item: LeTrackItem) -> Dir:
        return Dir.CODE_U
