from edge.support import fs, sh
from edge.webserver.sysconf.api_datetime.gpsd import GpsdConfTweaker
from edge.webserver.sysconf.api_datetime.types import NtpSourceConf, GnssSourceConf


class _ChronyConfBuilder:
    CONF_DIR = '/etc/chrony/conf.d'
    SOURCES_DIR = '/etc/chrony/sources.d'

    def _encode_ntp_source(self, ntp_source: NtpSourceConf) -> bool:
        conf_lines = list[str]()

        if ntp_source.get('autosync', False):
            for server_conf in ntp_source['servers']:
                address = server_conf['address']
                if server_conf['prefered']:
                    conf_lines.append(f'server {address} iburst prefer')
                else:
                    conf_lines.append(f'server {address} iburst')

        conf_path = fs.join_path(self.SOURCES_DIR, 'edge.sources')
        with open(conf_path, 'w') as text_file:
            text_file.write('\n'.join(conf_lines))

        return bool(conf_lines)

    def _encode_gnss_source(self, gnss_source: GnssSourceConf) -> None:
        conf_lines = list[str]()

        if gnss_source.autosync:
            assert gnss_source.gnss_enpoint is not None
            conf_lines.append(f'refclock SHM 0 offset {gnss_source.gnss_offset} delay 0.05 refid GNSS noselect')

            if gnss_source.pps_enpoint:
                pps_endpoint_suffix = gnss_source.pps_enpoint.split('/')[-1]
                pps_offset = gnss_source.pps_offset
                conf_lines.append(f'refclock SOCK /var/run/chrony.{pps_endpoint_suffix}.sock offset {pps_offset} refid PPS')

        conf_path = fs.join_path(self.CONF_DIR, 'edge.conf')
        with open(conf_path, 'w') as text_file:
            text_file.write('\n'.join(conf_lines))

    def build(self, ntp_source: NtpSourceConf, gnss_source: GnssSourceConf) -> bool:
        ntp_autosync = self._encode_ntp_source(ntp_source)
        self._encode_gnss_source(gnss_source)
        return ntp_autosync or gnss_source.autosync


def setup_chronyd(ntp_source: NtpSourceConf, gnss_source: GnssSourceConf) -> None:
    autosync = _ChronyConfBuilder().build(ntp_source, gnss_source)
    GpsdConfTweaker.tweak(gnss_source.endpoints)

    sh.run_silent('systemctl stop chrony')
    sh.run_silent('systemctl disable chrony')
    if autosync:
        sh.run_silent('systemctl enable chrony')
        sh.run_silent('systemctl start chrony')

    sh.run_silent('systemctl restart gpsd')
