import os

from edge import eventlog
from edge.support import fs

class GpsdConfTweaker:
    GPSD_DEB__CONF_PATH = '/etc/default/gpsd'

    @classmethod
    def _get_conf_path(cls):
        if fs.path_exists(cls.GPSD_DEB__CONF_PATH):
            return cls.GPSD_DEB__CONF_PATH
        else:
            eventlog.log_e('fail: unsupported')
            raise NotImplementedError

    @classmethod
    def tweak(cls, devices: list[str]) -> None:
        conf_path = cls._get_conf_path()

        with open(conf_path, 'r') as text_file:
            curr_lines = text_file.readlines()

        next_lines: list[str] = []
        for line in curr_lines:
            if line.startswith('DEVICES='):
                next_lines.append(f'DEVICES="{" ".join(devices)}"\n')
            elif line.startswith('GPSD_OPTION='):
                next_lines.append('GPSD_OPTIONS="-n"\n')
            else:
                next_lines.append(line)

        with open(conf_path, 'w') as text_file:
            text_file.write(''.join(next_lines))


def map_enpoint_gnss_to_pps(gnss_endpoint: str) -> str | None:
    """
    :param gnss_endpoint:
        /dev/ttyS3
    :return:
        /dev/pps1
    """
    if not gnss_endpoint:
        eventlog.log_e_param('gnss_endpoint')
        raise NotImplementedError

    pps_root_dir = '/sys/devices/virtual/pps'
    if not os.path.exists(pps_root_dir):
        return

    #
    # cat /sys/devices/virtual/pps/pps1/path
    # /dev/ttyS3
    subdir_names = os.listdir(pps_root_dir)
    for subdir_name in subdir_names:
        file_path = fs.join_path(pps_root_dir, subdir_name, 'path')
        with open(file_path, 'r') as text_file:
            lines = text_file.readlines()

        for line in lines:
            if line.strip() == gnss_endpoint:
                return f'/dev/{subdir_name}'
