from typing import Final, TypedDict, NotRequired


class SourceConf(TypedDict):
    type: str
    autosync: NotRequired[bool]


class GnssSourceConf:
    def __init__(self,
                 autosync: bool = False,
                 gnss_enpoint: str | None = None,
                 pps_enpoint: str | None = None,
                 gnss_offset: float = 0.0,
                 pps_offset: float = 0.0) -> None:

        self.autosync: Final[bool] = autosync

        self.gnss_enpoint: Final[str | None] = gnss_enpoint
        self.pps_enpoint: Final[str | None] = pps_enpoint

        self.gnss_offset: Final[float] = gnss_offset
        self.pps_offset: Final[float] = pps_offset

    @property
    def endpoints(self) -> list[str]:
        return [x for x in [self.gnss_enpoint, self.pps_enpoint] if x is not None]


class NtpServerConf(TypedDict):
    address: str
    prefered: bool


class NtpSourceConf(SourceConf):
    servers: list[NtpServerConf]
