from edge.webserver.sysconf.dgstats.camera import CamDgStats
from edge import eventlog
from edge.support import fs
from edge.support.template import FormatTemplate
from edge.types.exceptions import *
from edge.webserver.sysconf import edge_creator as creator
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.utype import *


# noinspection PyClassHasNoInit
class HttpStreamingApi:
    @staticmethod
    def load_unit(uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_HTTP_STREAMING)

        cam_dgstats = CamDgStats.create(unit_umod.pid)
        # cam_dgstats.assert_stream_active()
        cam_stats_jnode = cam_dgstats.get_stats()

        unit_uconf = unit_umod.load_conf()
        return {
            'type': UTYPE_HTTP_STREAMING,
            'conf': unit_uconf['rw'],
            'stats': cam_stats_jnode
        }

    @classmethod
    def save_unit(cls, uref: str, new_uconf: dict) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_HTTP_STREAMING)

        cam_dgstats = CamDgStats.create(unit_umod.pid)
        # cam_dgstats.assert_stream_active()

        new_uconf_conf = new_uconf.get('conf')

        def validate_uiconf_pub():
            fs.validate_json(new_uconf_conf, unit_umod.schema_path)

            # validate 'format' jnodes from json
            footer_rows = new_uconf_conf['live']['footer']['rows']
            if cls._validate_format(footer_rows) is False:
                eventlog.log_d('fail: ParamsError')
                raise ParamsError

        validate_uiconf_pub()

        restart_setupstore = []
        unit_uconf = unit_umod.load_conf()
        if unit_uconf['rw'] != new_uconf_conf:

            if unit_uconf['rw']['enabled'] != new_uconf_conf['enabled']:
                restart_setupstore.append(True)

            unit_uconf['rw'] = new_uconf_conf
            urefs_l0_l1 = rootinfo.get_linked_urefs(unit_umod.ref)
            creator.generate(unit_umod.create_backup_dirpath(), urefs_l0_l1, {unit_umod.ref: unit_uconf})

        if restart_setupstore:
            dg.LifeCycle.sighup_ss_dgmod()

        cam_stats_jnode = cam_dgstats.get_stats()
        return {
            'type': UTYPE_HTTP_STREAMING,
            'conf': new_uconf_conf,
            'stats': cam_stats_jnode
        }

    @staticmethod
    def _validate_format(jnode: list) -> bool:
        """
        :type jnode: list
        :rtype: bool
        """
        key_to_true_map = {
            'datetime': True,
            'datetime-ms': True,
            'post-name': True,
            'geolocation': True,
            'vendor': True,
            'model': True,
            'serial': True
        }

        for row in jnode:
            for col in row['cols']:
                try:
                    FormatTemplate(template=col['format']).substitute(key_to_true_map)
                except KeyError as x:
                    eventlog.log_d('fail: invalid key ({})'.format(str(x)))
                    return False

        return True
