from edge import eventlog
from edge.support import fs
from edge.types.exceptions import *
from edge.webserver.sysconf.dgstats.camera import CamDgStats
from edge.webserver.sysconf.uimod import rootinfo, unet
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.utype import *

from psuctl import Lens


# noinspection PyClassHasNoInit
class LensApi:
    @classmethod
    def load_unit(cls, uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_LENS)

        cam_dgstats = CamDgStats.create(unit_umod.pid)
        #cam_dgstats.assert_stream_active()

        root_uinfo = rootinfo.decode_info().root
        term_uinfo = unet.find_parent_r(root_uinfo, unit_uref=uref, parent_utype=UTYPE_TERMINAL)
        psuctl_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_PSUCTL)
        psuctl_uref = psuctl_uinfo['ref']

        psuctl_lens = Lens(psuctl_uref)
        _, max_iris = psuctl_lens.iris_limits
        _, max_focus = psuctl_lens.focus_limits

        unit_uconf = unit_umod.load_conf()

        reply = {
            'type': UTYPE_LENS,
            'conf': unit_uconf['rw'],
            'limits': {
                'iris': max_iris,
                'focus': max_focus
            },
            'stats': cam_dgstats.get_stats()
        }

        return reply

    @classmethod
    def save_unit(cls, uref: str, new_uconf: dict) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_LENS)

        cam_dgstats = CamDgStats.create(unit_umod.pid)
        #cam_dgstats.assert_stream_active()

        new_uconf_conf = new_uconf.get('conf')
        fs.validate_json(new_uconf_conf, unit_umod.schema_path)

        unit_uconf = unit_umod.load_conf()
        unit_uconf_pub = unit_uconf['rw']

        root_uinfo = rootinfo.decode_info().root
        term_uinfo = unet.find_parent_r(root_uinfo, unit_uref=uref, parent_utype=UTYPE_TERMINAL)
        psuctl_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_PSUCTL)
        psuctl_uref = psuctl_uinfo['ref']

        psuctl_lens = Lens(psuctl_uref)
        _, max_iris = psuctl_lens.iris_limits
        _, max_focus = psuctl_lens.focus_limits

        if unit_uconf_pub != new_uconf_conf:
            old_iris = unit_uconf_pub['iris']
            old_focus = unit_uconf_pub['focus']

            new_iris = new_uconf_conf['iris']
            new_focus = new_uconf_conf['focus']
            try:
                with psuctl_lens:
                    psuctl_lens.rotate_iris(new_iris, old_iris)
                    psuctl_lens.rotate_focus(new_focus, old_focus)

            except InternalError:
                eventlog.log_e(f'fail: InternalError')
                raise InternalError

            unit_uconf['rw'] = new_uconf_conf
            unit_umod.save_conf(unit_uconf)
            # uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            # from edge.webserver.sysconf.edge_creator import generate
            # generate(uimod.create_backup_dirpath(), uirefs, {uimod.ref: uiconf})

        reply = {
            'type': UTYPE_LENS,
            'conf': new_uconf_conf,
            'limits': {
                'iris': max_iris,
                'focus': max_focus
            },
            'stats': cam_dgstats.get_stats()
        }

        return reply
