from edge import eventlog
from edge.types.exceptions import *

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod import unet
from edge.webserver.sysconf.uimod.umod2 import Umod2
from edge.webserver.sysconf.uimod.uref import Uref2
from edge.webserver.sysconf.uimod.urole import *
from edge.webserver.sysconf.uimod.utype import *


class PsuctlApi:
    @staticmethod
    def make_service(term_uref: str, factory_conf: dict) -> None:
        root_unet = rootinfo.decode_info()
        root_info, root_links = root_unet.root, root_unet.links

        terminal_info = unet.find_unit_r(root_info, uref=term_uref)
        if terminal_info is None:
            eventlog.log_d(f'fail: NotFoundError ({UTYPE_TERMINAL})')
            raise NotFoundError

        uref_gen = rootinfo.UrefGenerator(terminal_info)
        unit_uref: Uref2 = uref_gen.create_unit_uref2(UTYPE_PSUCTL)
        unit_uconf = UiConf2(ph.build_term_unit_uconf(term_uref, unit_uref.type, unit_uref.ref, factory_conf))

        unit_umod = Umod2(unit_uref.ref)
        unit_umod.install(unit_uconf, unit_uref.ref)

        if terminal_info.get('units') is None:
            terminal_info['units'] = []

        terminal_info['units'].append({
            'ref': unit_umod.ref,
            'type': unit_umod.type,
            'name': unit_umod.ref,
            'role': UROLE_SERVICE,
            'internal': True
        })

        # add links
        urefs_mv_rtsp_ra = unet.find_urefs_r(root_info, utypes=[UTYPE_MV_CAMERA, UTYPE_RTSP_CAMERA, UTYPE_RADAR])
        for uref in urefs_mv_rtsp_ra:
            root_links.append((uref, unit_umod.ref))

        rootinfo.encode_info(root_unet)

    @staticmethod
    def drop_service(uref: str) -> dict:
        unit_umod = Umod2(uref)
        try:
            unit_umod.assert_type(UTYPE_PSUCTL)
            unit_umod.assert_ref()
        except NotFoundError:
            eventlog.log_d('fail: NotFoundError')
            raise

        root_info, _ = rootinfo.drop_unit_r(unit_umod.ref)
        unit_umod.uninstall()

        return {
            'info': [root_info]
        }
