from edge import eventlog
from edge.support import fs
from edge.types.exceptions import *
from edge.webserver.sysconf import edge_creator as creator
from edge.webserver.sysconf.dgstats.camera import CamDgStats
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.utype import *


# noinspection PyClassHasNoInit
class RadarApi:
    @staticmethod
    def load_unit(uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_RADAR)

        unit_uconf = unit_umod.load_conf(default=None)
        if unit_uconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        stats_jnode = CamDgStats.create(unit_umod.pid).get_stats()

        reply = {
            'type': UTYPE_RADAR,
            'conf': unit_uconf['rw']['conf'],
            'stats': stats_jnode
        }
        return reply

    @staticmethod
    def save_unit(uref: str, new_uconf: dict) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_RADAR)

        if unit_umod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        new_uconf_conf = new_uconf.get('conf')
        fs.validate_json(new_uconf_conf, unit_umod.schema_path)

        unit_uconf = unit_umod.load_conf()
        if unit_uconf['rw']['conf'] != new_uconf_conf:
            unit_uconf['rw']['conf'] = new_uconf_conf

            uirefs_l0_l1 = rootinfo.get_linked_urefs(unit_umod.ref)
            creator.generate(unit_umod.create_backup_dirpath(), uirefs_l0_l1, {unit_umod.ref: unit_uconf})

        stats_jnode = CamDgStats.create(unit_umod.pid).get_stats()
        return {
            'type': UTYPE_RADAR,
            'conf': new_uconf_conf,
            'stats': stats_jnode
        }
