from edge import eventlog
from edge.support import fs
from edge.support.mask import MaskRegion
from edge.types.exceptions import *
from edge.webserver.sysconf import edge_creator as creator
from edge.webserver.sysconf.dgstats.camera import CamDgStats
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.utype import *


# noinspection PyClassHasNoInit
class RoadwayApi:
    @staticmethod
    def load_unit(uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_ROADWAY)

        cam_dg_stats = CamDgStats.create(unit_umod.pid)
        #cam_dg_stats.assert_stream_active()
        cam_stats_jnode = cam_dg_stats.get_stats()

        unit_uconf = unit_umod.load_conf()

        return {
            'type': UTYPE_ROADWAY,
            'conf': unit_uconf['rw']['args'],
            'stats': cam_stats_jnode
        }

    @staticmethod
    def save_unit(uref: str, new_uconf: dict) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_ROADWAY)

        cam_dg_stats = CamDgStats.create(unit_umod.pid)
        #cam_dg_stats.assert_stream_active()
        cam_stats_jnode = cam_dg_stats.get_stats()

        # get image wh for pipeline/ui_pid/videosource/0
        sensor_wh = CamDgStats.create(unit_umod.pid).stream_wh

        new_uconf_conf = new_uconf.get('conf')

        def validate_uiconf():
            fs.validate_json({'args': new_uconf_conf}, unit_umod.schema_path)

            try:
                # validate self-intersection of lanes
                entry_lanes = new_uconf_conf['lanes']
                for lane_jnode in entry_lanes:
                    mask_region = MaskRegion(lane_jnode['mask'])
                    if not mask_region.test_valid():
                        eventlog.log_d('fail: ParamsError (invalid road-mask)')
                        raise ParamsError

            except (ValueError, IndexError):
                eventlog.log_d('fail: ParamsError (road-mask contains less then 3 vertexes)')
                raise ParamsError

        validate_uiconf()

        unit_uconf = unit_umod.load_conf()
        unit_uconf_pub = unit_uconf['rw']
        if unit_uconf_pub['args'] != new_uconf_conf:
            unit_uconf_pub['args'] = new_uconf_conf
            uirefs_l0_l1 = rootinfo.get_linked_urefs(unit_umod.ref)
            creator.generate(unit_umod.create_backup_dirpath(), uirefs_l0_l1, {unit_umod.ref: unit_uconf})

        return {
            'type': UTYPE_ROADWAY,
            'conf': new_uconf_conf,
            'stats': cam_stats_jnode
        }
