import os

from typing import Callable

from edge.support import fs


# noinspection PyClassHasNoInit
class SsDgconf:
    CONF_PATH = '/etc/edge/setupstore/main.conf'
    CONF_TMP_PATH = CONF_PATH + '.tmp'

    @classmethod
    def load_conf(cls, **kwargs) -> dict:
        """
        :raises FsError:
        """
        return fs.load_json3(cls.CONF_PATH, **kwargs)

    @classmethod
    def save_conf(cls, conf_jnode: dict) -> None:
        fs.write_json_file(conf_jnode, cls.CONF_PATH, pretty=True)

    @classmethod
    def set_bscache_spool_size(cls, size: int) -> None:
        dgconf = cls.load_conf()
        dgconf_groups: list[dict] = dgconf['groups']
        bscache_group: dict = [x for x in dgconf_groups if x['name'] == 'bs-cache'][0]
        bscache_group['spool-size'] = size

        cls.save_conf(dgconf)

    @classmethod
    def append_ss_group(cls, term_pid: str, dgconf_dirs: list[str], spool_size: int) -> None:
        dgconf = cls.load_conf()

        dgconf['groups'].append({
            'name': f'pipeline-{term_pid}',
            'spool-size': spool_size,
            'modules': dgconf_dirs
        })

        cls.save_conf(dgconf)

    @classmethod
    def remove_ss_group(cls, term_pid: str) -> None:
        dgconf = cls.load_conf()

        group_name = f'pipeline-{term_pid}'
        dgconf_groups = dgconf['groups']
        group_to_remove = [x for x in dgconf_groups if x['name'] == group_name]
        if len(group_to_remove) == 1:
            dgconf_groups.remove(group_to_remove[0])
            dgconf['groups'] = dgconf_groups
            cls.save_conf(dgconf)

    @classmethod
    def foreach_ss_module(cls,
                          handler: Callable[[str, str, str, str], None],
                          dgmod_types: set[str] | None = None) -> None:
        dgconf = cls.load_conf()
        dgconf_groups = dgconf['groups']

        for group in dgconf_groups:
            for dgconf_dirpath in group['modules']:
                dgconf_dirname = os.path.basename(os.path.normpath(dgconf_dirpath))
                dgmod_type, dgmod_id = dgconf_dirname.split('.')
                if dgmod_types and dgmod_type not in dgmod_types:
                    continue

                handler(group['name'], dgmod_type, dgmod_id, dgconf_dirpath)
