from edge.support import fs


class MainConfBuilder:
    def __init__(self, gamma_limits: list, target_median: list) -> None:
        self._jnode = {
            'Analyzer': {
                'ParamSet': {
                    'Target': {
                        'Median': target_median,
                        'SteadyState': {
                            'Unstable': 5,
                            'PreStable': 15,
                            'Stable': 30
                        }
                    },
                    'Sweep': {
                        'Gamma': gamma_limits
                    }
                }
            },
            'FileApi': {
                'AddSceneImgReact': {},
                'SceneControlEvent': []
            }
        }

    def setup_ir_control(self) -> None:
        self._jnode['Analyzer']['ParamSet']['Sweep']['IR'] = [1, 1, 1]

    def setup_gain_conf(self, gain_conf: list[float]) -> None:
        self._jnode['Analyzer']['ParamSet']['Sweep']['Gain'] = gain_conf

    def setup_exp_time_conf(self, exp_time_conf: list[int]) -> None:
        self._jnode['Analyzer']['ParamSet']['Sweep']['ExpTime'] = exp_time_conf

    def setup_add_scene_img_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode['FileApi']['AddSceneImgReact']['Url'] = fs.join_path(spool_dir, react_dir)

    def setup_add_scene_control_event(self, event_dir: str) -> None:
        self._jnode['FileApi']['SceneControlEvent'].append(event_dir)

    def build(self) -> dict:
        return self._jnode
