from copy import deepcopy

from edge.types.identity import Identity


def create_unit__anpr__anpr(unit_name: str, pri_conf: dict, pub_conf: dict,
                            image_wh: tuple[int, int]) -> tuple[str, dict]:

    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    plate_detector_jnode = deepcopy(pub_conf['plate-detector'])
    plate_detector_jnode['extra-ranges-analyse'] = pri_conf.get('extra-ranges-analyse') or False
    plate_detector_jnode['extra-angle-analyse'] = pri_conf.get('extra-angle-analyse') or False
    plate_detector_jnode['precise-analyse'] = pri_conf.get('precise-analyse') or False

    return unit_name, {
        'bundle': ['libunity_anpr', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': 'anpr',
                'name': uiref.ref,
                'conf': {
                    'engine': {
                        'log-settings': False,
                        'license-group': int(pub_conf['license-group']),
                        'license-count': pub_conf['license-count'],
                        'image-tout': pri_conf.get('image-tout') or 250,
                        'plate-detector': plate_detector_jnode,
                        'plate-recognizer': pub_conf['plate-recognizer'],
                        'plate-tracker': {
                            'lost-tout': pub_conf['plate-tracker']['lost-tout'],
                            'size-min': pub_conf['plate-tracker'].get('min-size', 4)
                        }
                    },
                    'stream': {
                        'wh': image_wh,
                    },
                    'postprocess': pri_conf.get('postprocess') or 1000,
                    'activation': pri_conf.get('activation') or 'image'
                }
            }
        ]
    }
