from typing import Final

from edge.webserver.sysconf.dgmod.mg.dgconf.mgd_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.mg.dgconf.mgd_unit import UnitConfBuilder
from edge.webserver.sysconf.dgmod.mg.uiconf import IUiConfProvider2


class MvCameraUiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self,
                       unit_builder: UnitConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: dict = self._uiconf['ro']
        pub_conf: dict = self._uiconf['rw']

        vs_jnode = pub_conf['videostream']
        vs_control_jnode = vs_jnode['control']
        vs_sensor_jnode = vs_jnode['resolution']
        vs_sensor_fps = vs_jnode['fps']

        raw_format = pri_conf['raw-format']

        unit_builder.setup_vst_busstore(
            'mg{0}-mg{0}-store-raw'.format(unit_builder.uimod_id),
            vs_sensor_fps * 2, vs_sensor_fps * 2 + 5, raw_format)

        unit_builder.setup_driver(pub_conf['vendor'], pub_conf['model'], pub_conf['serial'])
        unit_builder.setup_driver_cam_vs_acq(
            vs_sensor_jnode['w'], vs_sensor_jnode['h'],
            vs_sensor_jnode['x'], vs_sensor_jnode['y'],
            vs_sensor_fps, vs_jnode['rotate180'])

        if vs_control_jnode['profile'] == 'manual':
            unit_builder.setup_driver_cam_vs_sweep_manual(
                vs_control_jnode['exposure-manual'],
                vs_control_jnode['gain-manual'],
                vs_control_jnode['gamma-manual'])

        elif vs_control_jnode['profile'] == 'auto-70-150':
            unit_builder.setup_driver_cam_vs_sweep_manual(
                vs_control_jnode['exposure-auto-min'],
                vs_control_jnode['gain-auto-min'],
                pri_conf.get('gamma-def', 0.7))

        ir_list = pri_conf.get('irbox') or list()
        for ir_jnode in ir_list:
            unit_builder.setup_driver_cam_pinbox_ir(
                ir_jnode.get('source', 'ExposureActive'),
                ir_jnode.get('exposure-ratio', 100),
                ir_jnode['pin'],
                ir_jnode['inverter'])

        unit_builder.setup_driver_cam_misc(pri_conf['transfer-data-size'])

        if raw_format == 'MONO8T':
            unit_builder.setup_vst_bus(['mg{0}-mcgst{0}-scene'.format(unit_builder.uimod_id)])

            if vs_control_jnode['profile'] == 'auto-70-150':
                unit_builder.setup_vst_bus(['mg{0}-gc{0}-scene'.format(unit_builder.uimod_id)])

        else:
            unit_builder.setup_vst_bus(['mg{0}-rc{0}-scene'.format(unit_builder.uimod_id)])

        halt_lines = pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
