from typing import Final

from edge.webserver.sysconf.dgmod.ra.ra_api import ApiConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.ra.uiconf import IUiConfProvider2


class LocationUiConfProvider(IUiConfProvider2):
    def __init__(self, ra_uiconf: dict, repro_uiconf: dict, uiconf_changed: bool) -> None:
        self._repro_uiconf: Final[dict] = repro_uiconf
        self._ra_uiconf: Final[dict] = ra_uiconf
        self._uiconf_changed: Final[bool] = uiconf_changed

    def setup_builders(self,
                       api_builder: ApiConfBuilder | None,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        ra_pri_conf: dict = self._ra_uiconf['ro']

        ra_device = ra_pri_conf['device']
        driver = ra_device.get('driver') or 'libdriver-smartmicro'
        endpoint = ra_device.get('uri') or 'ummr0a:///dev/ttyS0'
        main_builder.setup_device(driver, endpoint)
        main_builder.setup_scene_event_zone(ra_device.get('zone'))

        if api_builder is not None:
            repro_conf_jnode: dict = self._repro_uiconf['rw']['conf']
            ra_conf_jnode: dict = self._ra_uiconf['rw']['conf']

            cam_jnode = repro_conf_jnode['camera']['extrinsic']
            cam_position_jnode = cam_jnode['position']
            camera_f = cam_position_jnode['forward']
            camera_l = cam_position_jnode['left']
            camera_u = cam_position_jnode['up']

            radar_position_jnode = ra_conf_jnode['radar']['rel-position']
            radar_f = camera_f + radar_position_jnode['forward']
            radar_l = camera_l + radar_position_jnode['left']
            radar_u = camera_u + radar_position_jnode['up']

            api_builder.setup_unit_position(camera_f, camera_l, camera_u, radar_f, radar_l, radar_u)

            cam_rotation_jnode = cam_jnode['rotation']
            cam_fl = cam_rotation_jnode['azimuth']
            cam_fu = cam_rotation_jnode['elevation']
            cam_lu = cam_rotation_jnode['roll']
            api_builder.setup_camera_orientation(cam_fl, cam_fu, cam_lu)

            radar_rotation_jnode = ra_conf_jnode['radar']['rel-rotation']
            radar_fl = cam_fl + radar_rotation_jnode['azimuth']
            radar_fu = cam_fu + radar_rotation_jnode['elevation']
            radar_lu = cam_lu + radar_rotation_jnode['roll']
            api_builder.setup_radar_orientation(radar_fl, radar_fu, radar_lu)

        halt_lines: list[str] | None = ra_pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
