from typing import Final

from edge.webserver.sysconf.dgmod.ra.ra_api import ApiConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.ra.uiconf import IUiConfProvider2


class RtspCameraUiConfPorvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, cam_uiconf: dict, repro_uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._rtsp_cam_uiconf: Final[dict] = cam_uiconf
        self._repro_uiconf: Final[dict] = repro_uiconf

    def setup_builders(self,
                       api_builder: ApiConfBuilder | None,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:
        main_builder.setup_strobe_react(f'rg{self._uimod_id}-ra{main_builder.uimod_id}-pipe', 0)

        if api_builder is not None:
            intrinsic_jnode: dict | None = self._repro_uiconf['rw']['conf']['camera'].get('intrinsic') or dict()
            if intrinsic_jnode is not None:
                frustum_jnode = intrinsic_jnode['frustum']

                pixel_size_mm: float = frustum_jnode['pixel-size'] * 0.001
                sensor_wh_px: tuple[int, int] = frustum_jnode['sensor-size']
                sensor_wh_mm: tuple[float, float] = sensor_wh_px[0] * pixel_size_mm, sensor_wh_px[1] * pixel_size_mm
                focal_dist_mm = frustum_jnode['focal-dist']

                api_builder.setup_camera(sensor_wh_px, sensor_wh_mm, focal_dist_mm)
