from typing import Final

from edge.support import fs


class MainConfBuilder:
    def __init__(self, uimod_id: str, spool_dir: str) -> None:
        self.uimod_id: Final[str] = uimod_id
        self._spool_dir: Final[str] = spool_dir

        self._jnode = {
            'FileApi': {
                'SceneEvent': list()
            },
            'codec': {
                'Scale': {
                    'MaxWidth': 0,
                    'MaxHeight': 0,
                    'Interpolation': 1
                },
                'Encode': {
                    'UsedThreadMax': 1
                }
            }
        }

    def setup_react(self, react_dir: str) -> None:
        self._jnode['FileApi'].update({
            'AddSceneReact': {
                'Dir': fs.join_path(self._spool_dir, react_dir)
            }
        })

    def setup_event_blobs(self, blob_dir: str) -> None:
        self._jnode['FileApi'].update({
            'Store': fs.join_path(self._spool_dir, blob_dir),
            'FileCountLow': 18,
            'FileCountHigh': 20,
        })

    def setup_event_args(self, args_dirs: list[str]) -> None:
        file_api_jnode: dict = self._jnode['FileApi']
        scene_event_jnode: list[str] = file_api_jnode['SceneEvent']

        for args_dir in args_dirs:
            scene_event_jnode.append(fs.join_path(self._spool_dir, args_dir))

    def build(self) -> dict:
        return self._jnode
