from typing import Final

from edge import eventlog
from edge.types.exceptions import NotFoundError
from edge.webserver.sysconf.dgmod.rn.rn_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rn.rn_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rn.uiconf import IUiConfProvider2


def _decode__tl_id(pri_jnode: dict) -> int:
    """
    :param pri_jnode:
        #/ro

    :raises NotFoundError:
    """
    params_jnode: dict | None = pri_jnode.get('params')
    if params_jnode is None:
        eventlog.log_d('fail')
        raise NotFoundError()

    controller_jnode: dict | None = params_jnode.get('controller')
    if not controller_jnode:
        eventlog.log_d('fail')
        raise NotFoundError()

    tl0_jnode: dict = controller_jnode[0]
    tl0_ref: int | None = tl0_jnode.get('id')
    if tl0_ref is None:
        eventlog.log_d('fail')
        raise NotFoundError()

    return tl0_ref


class _UiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self, main_builder: MainConfBuilder, ss_builder: SetupStoreConfBuilder) -> None:
        pri_jnode: dict = self._uiconf['ro']

        try:
            tl_id: int = _decode__tl_id(pri_jnode)
        except NotFoundError:
            tl_id: int = 1

        main_builder.setup_trafficlight_react(
            'tl{0}-or{0}-track'.format(main_builder.uimod_id),
            tl_id)


RedlightUiConfProvider = _UiConfProvider
RailroadUiConfProvider = _UiConfProvider
