from typing import Final

from edge.webserver.sysconf.dgmod.rn.rn_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rn.rn_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rn.uiconf import IUiConfProvider2


class SpotSpeedUiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self, main_builder: MainConfBuilder, ss_builder: SetupStoreConfBuilder) -> None:
        pub_jnode: dict = self._uiconf['rw']
        args_jnode: dict = pub_jnode['args']
        rules_jnode: list[dict] = args_jnode['rules']

        # Do not render overspeed if spotspeed detector has more than 1 rule
        if len(rules_jnode) != 1:
            return None

        rule0_jnode: dict = rules_jnode[0]
        if not rule0_jnode['enabled']:
            return None

        rule0_limits_jnode: dict = rule0_jnode['limits']
        rule0_speed_limit: int = rule0_limits_jnode['speed-limit']
        rule0_max_overspeed: int = rule0_limits_jnode['max-overspeed']

        main_builder.setup_spotspeed(rule0_speed_limit, rule0_max_overspeed)
