from typing import Final

from edge.support import fs


class _StreamFpsBuilder:
    def __init__(self):
        self._cam_fps: int | None = None
        self._skip_rate: int | None = None

    def setup_cam_fps(self, cam_fps: int) -> None:
        self._cam_fps = cam_fps

    def setup_skip_rate(self, skip_rate: int) -> None:
        self._skip_rate = skip_rate

    def build(self) -> int:
        if self._cam_fps is None or self._skip_rate is None:
            return 0  # TODO: review this

        return self._cam_fps // self._skip_rate


class MainConfBuilder:
    def __init__(self, uimod_id: str, spool_dir: str) -> None:
        self.uimod_id: Final[str] = uimod_id
        self._spool_dir: Final[str] = spool_dir

        self._stream_fps_builder = _StreamFpsBuilder()

        self._jnode = {
            'Stream': {
            },
            'FileApi': {
                'AddSceneImgReact': {
                }
            },
            'Resolution': {
                'MaxHeight': 65535,
            }
        }

    def setup_react(self, react_dir: str) -> None:
        self._jnode['FileApi']['AddSceneImgReact'].update({
            'Url': fs.join_path(self._spool_dir, react_dir)
        })

    def setup_streamer__url(self, stream_url: str) -> None:
        self._jnode['Stream'].update({
            'Link': stream_url
        })

    def setup_streamer__udp_ports(self, min_port: int | None, max_port: int | None) -> None:
        self._jnode['Stream'].update({
            'UdpPorts': [min_port, max_port]
        })

    def setup_stream__prefer_gpu(self, prefer_gpu: bool) -> None:
        self._jnode.update({
            'UseGPU': prefer_gpu
        })

    def setup_stream__quality(self, quality: int) -> None:
        self._jnode.update({
            'Quality': quality
        })

    def setup_stream__cam_fps(self, fps: int) -> None:
        self._stream_fps_builder.setup_cam_fps(fps)

    def setup_stream__skip_rate(self, skip_rate: int) -> None:
        self._stream_fps_builder.setup_skip_rate(skip_rate)

    def setup_stream__wh(self, max_w: int, max_h: int = 65535) -> None:
        self._jnode.update({
            'Resolution': {
                'MaxWidth': max_w,
                'MaxHeight': max_h
            }
        })

    def build(self) -> dict:
        self._jnode.update({
            'Fps': self._stream_fps_builder.build()
        })

        return self._jnode
