from typing import Final

from edge.webserver.sysconf.dgmod.rs.rs_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rs.rs_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rs.uiconf import IUiConfProvider2


class RtspStreamingUiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self, main_builder: MainConfBuilder, ss_builder: SetupStoreConfBuilder) -> None:
        pri_jnode = self._uiconf['rw']

        streamer_jnode: dict = pri_jnode['streaming']
        streamer_tcp_port: int = streamer_jnode['tcp-port']
        streamer_transport: str = streamer_jnode['transport']

        if streamer_transport == 'rtp-over-ucast-udp-or-tcp':
            main_builder.setup_streamer__udp_ports(
                streamer_jnode['min-udp-port'],
                streamer_jnode['max-udp-port'])

        elif streamer_transport == 'rtp-over-tcp':
            main_builder.setup_streamer__udp_ports(None, None)

        else:
            raise NotImplementedError()

        stream_jnode: dict = pri_jnode['live']['videostream']
        stream_prefer_gpu: bool = not stream_jnode['sw-codec']
        stream_skip_rate: int = stream_jnode['skip-rate']
        stream_format: str = stream_jnode['format']
        stream_max_w: int = stream_jnode['frame-w']

        if stream_format == 'h264-cqp':
            main_builder.setup_streamer__url(f'rtsp://@:{streamer_tcp_port}/live-h264')
            main_builder.setup_stream__quality(stream_jnode['format-h264-cqp-compression'])
            main_builder.setup_stream__skip_rate(stream_skip_rate)
            main_builder.setup_stream__prefer_gpu(stream_prefer_gpu)
            main_builder.setup_stream__wh(stream_max_w)

        elif stream_format == 'h264-cbr':
            main_builder.setup_streamer__url(f'rtsp://@:{streamer_tcp_port}/live-h264')
            main_builder.setup_stream__quality(stream_jnode['format-h264-cbr-bitrate-max'])
            main_builder.setup_stream__skip_rate(stream_skip_rate)
            main_builder.setup_stream__prefer_gpu(stream_prefer_gpu)
            main_builder.setup_stream__wh(stream_max_w)

        elif stream_format == 'mjpeg':
            main_builder.setup_streamer__url(f'rtsp://@:{streamer_tcp_port}/live-mjpeg')
            main_builder.setup_stream__quality(stream_jnode['format-mjpeg-quality'])
            main_builder.setup_stream__skip_rate(stream_skip_rate)
            main_builder.setup_stream__prefer_gpu(stream_prefer_gpu)
            main_builder.setup_stream__wh(stream_max_w)

        else:
            raise NotImplementedError()

        main_builder.setup_react('or{0}-rs{0}-scene'.format(main_builder.uimod_id))
        ss_builder.setup_active(pri_jnode['enabled'])
