from typing import Final

from edge.support import fs


class _DeviceBuilder:
    def __init__(self):
        self._vendor: str | None = None
        self._params_video__controller: list[dict] = []
        self._params_video__endpoint_: str | None = None
        self._params_io: dict | None = None

    def setup_vendor(self, vendor: str) -> None:
        self._vendor = vendor

    def setup_params_video__traffic_light(
            self,
            ref: int,
            red_zone: list[int],
            amber_zone: list[int],
            green_zone: list[int]) -> None:

        self._params_video__controller.append({
            'id': ref,
            'type': 'traffic-light',
            'red-chan': 1,
            'red-zone': red_zone,
            'amber-chan': 2,
            'amber-zone': amber_zone,
            'green-chan': 3,
            'green-zone': green_zone
        })

    def setup_params_video__train_light(
            self,
            ref: int,
            red_zone1: list[int],
            red_zone2: list[int],
            threshold: float,
            bin_threshold: float,
            max_count: int) -> None:

        self._params_video__controller.append({
            'id': ref,
            'type': 'train-light',
            'red-chan': 1,
            'red-zone1': red_zone1,
            'red-zone2': red_zone2,
            'threshold': threshold,
            'bin-threshold': bin_threshold,
            'max-count': max_count
        })

    def setup_params_video__endpoint(self, react_dir: str) -> None:
        self._params_video__endpoint_ = react_dir

    def setup_params_io(self, params_jnode: dict | None) -> None:
        self._params_io = params_jnode

    def build(self) -> dict | None:
        if self._vendor is None:
            return None

        if self._vendor == 'zonechecker':
            if self._params_video__controller and self._params_video__endpoint_:
                return {
                    'vendor': self._vendor,
                    'params': {
                        'controller': self._params_video__controller,
                        'endpoint': self._params_video__endpoint_
                    }
                }

        elif self._params_io is not None:
            return {
                'vendor': self._vendor,
                'params': self._params_io
            }


class MainConfBuilder:
    def __init__(self, uimod_id: str, spool_dir: str) -> None:
        self.uimod_id: Final[str] = uimod_id
        self._spool_dir: Final[str] = spool_dir
        self._device_builder = _DeviceBuilder()

        self._jnode = {
            # "device": {
            #     "vendor": "komkon",
            #     "params": {
            #         "endpoint": "/dev/ttyUSB3",
            #         "controller": [
            #             {
            #                 "id": 1,
            #                 "red-chan": 1,
            #                 "amber-chan": 2
            #             }
            #         ]
            #     }
            # },
            'device': {
                'params': {
                    'controller': [

                    ],
                }
            },
            'changelog': {
                'entry-count': 1200,
                'entry-tout': 125,  # pass 600 for train-trafficlight
                'entry-age': 300
            },
            'track-event': [
            ],
            'monitoring': {
                'stat-file': fs.join_path(spool_dir, 'zabbix', f'tl{uimod_id}'),
                'stat-tout': 30
            }
        }

    def setup_track_event_args(self, args_dirs: list[str]) -> None:
        dirs_jnode: list[str] = self._jnode['track-event']

        for args_dir in args_dirs:
            dirs_jnode.append(fs.join_path(self._spool_dir, args_dir))

    def setup_delay(self, delay_usec: int) -> None:
        self._jnode['delay-usec'] = delay_usec

    def setup_device_vendor(self, vendor: str) -> None:
        self._device_builder.setup_vendor(vendor)

    def setup_device_video__traffic_light(
            self,
            ref: int,
            red_zone: list[int],
            amber_zone: list[int],
            green_zone: list[int]) -> None:

        self._device_builder.setup_params_video__traffic_light(ref, red_zone, amber_zone, green_zone)

    def setup_device_video__train_light(
            self,
            ref: int,
            red_zone1: list[int],
            red_zone2: list[int],
            threshold: float,
            bin_threshold: float,
            max_count: int) -> None:

        self._device_builder.setup_params_video__train_light(
            ref, red_zone1, red_zone2, threshold, bin_threshold, max_count)

    def setup_device_video__endpoint(self, args_dir: str) -> None:
        self._device_builder.setup_params_video__endpoint(fs.join_path(self._spool_dir, args_dir))

    def setup_device_io(self, params_jnode: dict | None) -> None:
        self._device_builder.setup_params_io(params_jnode)

    def build(self) -> dict:
        device_jnode: dict | None = self._device_builder.build()
        if device_jnode is not None:
            self._jnode['device'] = device_jnode

        return self._jnode
