from typing import Final

from edge import eventlog
from edge.types.exceptions import NotFoundError
from edge.webserver.sysconf.dgmod.tl.tl_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.tl.uiconf import IUiConfProvider2


def _decode__tl_id(pri_jnode: dict) -> int:
    """
    :param pri_jnode:
        #/ro

    :raises NotFoundError:
    """
    params_jnode: dict | None = pri_jnode.get('params')
    if params_jnode is None:
        eventlog.log_d('fail')
        raise NotFoundError()

    controller_jnode: dict | None = params_jnode.get('controller')
    if not controller_jnode:
        eventlog.log_d('fail')
        raise NotFoundError()

    tl0_jnode: dict = controller_jnode[0]
    tl0_ref: int | None = tl0_jnode.get('id')
    if tl0_ref is None:
        eventlog.log_d('fail')
        raise NotFoundError()

    return tl0_ref


class RailroadUiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self, main_builder: MainConfBuilder) -> None:
        pri_jnode: dict = self._uiconf['ro']
        pub_jnode: dict = self._uiconf['rw']

        try:
            tl_id: int = _decode__tl_id(pri_jnode)
        except NotFoundError:
            tl_id: int = 1

        delay_usec: int = pri_jnode.get('delay-usec') or 0
        main_builder.setup_delay(delay_usec)

        vendor: str = pri_jnode.get('vendor') or 'zonechecker'
        if vendor == 'zonechecker':
            tl_jnode: dict = pub_jnode['args']['scene']['area']['traffic-lights']
            red_zone1: list[int] = tl_jnode['red1']
            red_zone2: list[int] = tl_jnode['red2']

            threshold = pri_jnode.get('threshold', 0.1)
            bin_threshold = pri_jnode.get('bin-threshold', 0.1)
            max_count = pri_jnode.get('max-count', 6)

            main_builder.setup_device_vendor(vendor)
            main_builder.setup_device_video__train_light(
                tl_id, red_zone1, red_zone2, threshold, bin_threshold, max_count)

        else:
            params_jnode: dict | None = pri_jnode.get('params')

            main_builder.setup_device_vendor(vendor)
            main_builder.setup_device_io(params_jnode)
