from edge import eventlog
from edge.types.exceptions import NotFoundError, InternalError
from edge.webserver.sysconf.uiconf.roadway import RoadwayUconf
from edge.webserver.sysconf.uimod import unet
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.unet import UnitInfo
from edge.webserver.sysconf.uimod.utype import UTYPE_ROADWAY


class RoadwayUlimits:
    def __init__(self, root_uinfo: UnitInfo, term_uref: str) -> None:
        term_uinfo = unet.find_unit_r(root_uinfo, uref=term_uref)
        if term_uinfo is None:
            eventlog.log_e(f'fail: find_unit_r (uref:{term_uref})')
            raise NotFoundError

        roadway_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_ROADWAY)
        if roadway_uinfo is None:
            eventlog.log_e(f'fail: find_unit_r (utype:{UTYPE_ROADWAY})')
            raise InternalError

        roadway_uref = roadway_uinfo['ref']
        roadway_umod = Umod.create(roadway_uref)
        roadway_uconf = RoadwayUconf(roadway_umod.load_conf())

        self._roadway_uconf = roadway_uconf

    @property
    def lane_names(self) -> list[str]:
        return self._roadway_uconf.lane_names
