from PIL import Image
from typing import List

from edge.types.image import ImRgba
from edge.support.queue import JsonFileQueue, ImageFileQueue

from renderer.services.types.event import IEventEmitter, EventEmitterConf


class _EventEmitter(IEventEmitter):
    _STATUS_SUCCESS = 1
    _STATUS_FAILURE = 2

    def __init__(self, twisted_reactor, codec: str,
                 json_file_queue_list: List[JsonFileQueue],
                 image_file_queue: ImageFileQueue):

        self._twisted_reactor = twisted_reactor
        self._target_codec = codec
        self._json_file_queue_list = json_file_queue_list
        self._image_file_queue = image_file_queue

    def emit(self, scene_ts: int, scene_im: ImRgba) -> None:
        w, h = scene_im.wh
        with Image.frombuffer(
                'RGBA', (w, h + scene_im.dh), scene_im.buffer,
                'raw', 'RGBA', scene_im.stride, 1) as pil_img:

            image_ref = self._image_file_queue.add_image(scene_ts, pil_img)

            event_args_jnode = {
                'Ts': image_ref.ts,
                'Status': _EventEmitter._STATUS_SUCCESS,  # todo
                'SizeWh': pil_img.size,
                'Url': image_ref.url,
                'Codec': image_ref.codec
            }

            list(map(lambda q: q.add_jnode(event_args_jnode), self._json_file_queue_list))


def init_event_emitter(twisted_reactor, conf: EventEmitterConf) -> IEventEmitter:
    json_file_queue_list = [JsonFileQueue(queue_args) for queue_args in conf.json_store.json_queue_args_list]

    image_file_queue = ImageFileQueue(
        twisted_reactor, conf.im_store.codec,
        conf.im_store.image_queue_args.endpoint,
        conf.im_store.image_queue_args.min_file_count,
        conf.im_store.image_queue_args.max_file_count,
        conf.json_store.log_level)

    return _EventEmitter(twisted_reactor, conf.im_store.codec, json_file_queue_list, image_file_queue)
