import os
from typing import Dict

from edge import eventlog
from edge.types.exceptions import JnodeError
from renderer.services.types.event import ImStoreConf, ImQueueConf, JsonStoreConf, JsonQueueConf


def decode__im_store_conf(jnode: Dict) -> ImStoreConf:
    """
    :param jnode:
        main.conf#/image-store

    :raise JnodeError:
    """
    im_format = jnode.get('format')
    if im_format not in ['JPEG', 'RGBA']:
        eventlog.log_e_param('#/image-store')
        raise JnodeError

    im_max_count = jnode.get('max-file-count', 4)
    if not (int == type(im_max_count) and 4 <= im_max_count <= 8):
        eventlog.log_e_param('#/max-file-count')
        raise JnodeError

    endpoint = jnode.get('endpoint')
    if not (type(endpoint) in [str] and len(endpoint) < 64 and os.path.isabs(endpoint)):
        eventlog.log_e_param('#/endpoint')
        raise JnodeError

    im_q = ImQueueConf(endpoint, im_max_count - 2, im_max_count)
    return ImStoreConf(im_format, im_q)


def decode__json_store_conf(jnode: Dict, log_level: str) -> JsonStoreConf:
    """
    :param jnode:
        main.conf#/event-store
    :param log_level:

    :raise JnodeError:
    """
    max_file_count = jnode.get('max-file-count', 4)
    if not (int == type(max_file_count) and 4 <= max_file_count <= 8):
        eventlog.log_e_param('#/max-file-count')
        raise JnodeError

    q_args = [JsonQueueConf(endpoint, max_file_count - 5, max_file_count)
              for endpoint in jnode.get('endpoints') or list()]

    return JsonStoreConf(q_args, log_level)
