from edge import eventlog
from edge.types.exceptions import JnodeError
from renderer.services.types.jnode.tl import decode__tl_conf
from renderer.services.types.renderer import LrAnnoConf, RaAnnoConf, RendererConf
from renderer.services.types.tl import TlConf


def decode__lr_anno_conf(jnode: dict) -> LrAnnoConf:
    """
    :param jnode:
        main.conf#/Renderer/profile/vehicle/lr

    :raise JnodeError:
    """
    modes = [LrAnnoConf.MODE_NEVER, LrAnnoConf.MODE_ALWAYS]

    # unpack #/plate
    plate_mode = jnode.get('plate', LrAnnoConf.MODE_NEVER)
    if plate_mode not in modes:
        eventlog.log_e_param('#/plate')
        raise JnodeError

    # unpack #/trace
    trace_mode = jnode.get('trace', LrAnnoConf.MODE_NEVER)
    if trace_mode not in modes:
        eventlog.log_e_param('#/trace')
        raise JnodeError

    # unpack #/dir
    dir_mode = jnode.get('dir', LrAnnoConf.MODE_NEVER)
    if dir_mode not in modes:
        eventlog.log_e_param('#/dir')
        raise JnodeError

    # unpack #/lane
    lane_mode = jnode.get('lane', LrAnnoConf.MODE_NEVER)
    if lane_mode not in modes:
        eventlog.log_e_param('#/lane')
        raise JnodeError

    # unpack #/center
    center_mode = jnode.get('center', LrAnnoConf.MODE_NEVER)
    if center_mode not in modes:
        eventlog.log_e_param('#/center')
        raise JnodeError

    # unpack #/vehicle
    vehicle_bbox_mode = jnode.get('vehicle-bbox', LrAnnoConf.MODE_NEVER)
    if vehicle_bbox_mode not in modes:
        eventlog.log_e_param('#/vehicle-bbox')
        raise JnodeError

    vehicle_cuboid_mode = jnode.get('vehicle-cuboid', LrAnnoConf.MODE_NEVER)
    if vehicle_cuboid_mode not in modes:
        eventlog.log_e_param('#/vehicle-cuboid')
        raise JnodeError

    return LrAnnoConf(dir_mode, lane_mode, plate_mode, trace_mode, center_mode, vehicle_bbox_mode, vehicle_cuboid_mode)


def decode__ra_anno_conf(jnode: dict, yellow_speed: int, red_speed: int) -> RaAnnoConf:
    """
    :param jnode:
        main.conf#/Renderer/profile/vehicle/ra
    :param yellow_speed:
    :param red_speed:

    :raise JnodeError:
    """
    modes = [RaAnnoConf.MODE_NEVER, RaAnnoConf.MODE_IF_PLATE, RaAnnoConf.MODE_ALWAYS]

    # unpack #/target
    target_mode = jnode.get('target', RaAnnoConf.MODE_NEVER)
    if target_mode not in modes:
        eventlog.log_e_param('#/target')
        raise JnodeError

    # unpack #/vehicle
    vehicle_mode = jnode.get('vehicle', RaAnnoConf.MODE_NEVER)
    if vehicle_mode not in modes:
        eventlog.log_e_param('#/vehicle')
        raise JnodeError

    # unpack #/speed
    speed_mode = jnode.get('speed', RaAnnoConf.MODE_NEVER)
    if speed_mode not in modes:
        eventlog.log_e_param('#/speed')
        raise JnodeError

    # unpack #/plate
    dir_mode = jnode.get('dir', RaAnnoConf.MODE_NEVER)
    if dir_mode not in modes:
        eventlog.log_e_param('#/dir')
        raise JnodeError

    # unpack #/len
    len_mode = jnode.get('len', RaAnnoConf.MODE_NEVER)
    if len_mode not in modes:
        eventlog.log_e_param('#/len')
        raise JnodeError

    return RaAnnoConf(yellow_speed, red_speed, len_mode, dir_mode, target_mode, speed_mode, vehicle_mode,
                      radar_present=True)


def decode__renderer_conf(jnode: dict, yellow_speed: int, red_speed: int, lr_store_tout: int) -> RendererConf:
    """
    :param jnode: main.conf#/Renderer/profile/vehicle
    :param yellow_speed:
    :param red_speed:
    :param lr_store_tout:
    """
    vehilce_jnode = jnode.get('vehicle') or dict()
    lr_jnode = vehilce_jnode.get('lr')
    lr_anno = decode__lr_anno_conf(lr_jnode) if lr_jnode else LrAnnoConf()

    ra_jnode = vehilce_jnode.get('ra')
    if ra_jnode:
        ra_anno = decode__ra_anno_conf(ra_jnode, yellow_speed, red_speed)
    else:
        ra_anno = RaAnnoConf(yellow_speed, red_speed, radar_present=False)

    tl_args: list[TlConf] = decode__tl_conf(jnode)

    return RendererConf(lr_anno, ra_anno, tl_args, lr_store_tout)
