import ujson

from typing import Any, Dict, List

from edge import eventlog
from edge.types.identity import Identity
from edge.types.exceptions import JnodeError

from speedcontrol.conf.detectors import DetectorsConf
from speedcontrol.detectors.types.detector import IDetector, DetectorUnitInfo


def decode__detectors_conf(detectors_jnode: Dict, def_footer_vars: Dict[str, Any]) -> DetectorsConf:
    def decode_detectors(type_name: str, conf_key: str) -> List[DetectorUnitInfo]:

        conf_path_list: List[str] = detectors_jnode.get(conf_key, [])
        if type(conf_path_list) not in (list,):
            eventlog.log_e_param(f'#/detectors/{conf_key}')
            raise JnodeError

        elif len(conf_path_list) > 64:
            eventlog.log_e_param(f'#/detectors/{conf_key}')
            raise JnodeError

        detector_id = 0
        result: List[DetectorUnitInfo] = []
        for conf_path in conf_path_list:
            with open(conf_path) as json_file:
                detector_jnode = ujson.load(json_file)

            identity = Identity.create(type_name, detector_id)
            result.append(DetectorUnitInfo(identity, detector_jnode, def_footer_vars))
            detector_id += 1

        return result

    base_confs = decode_detectors(IDetector.TYPE_BASE, 'transit.conf')
    transit_confs = decode_detectors(IDetector.TYPE_TRANSIT, 'transit.conf')
    buslane_confs = decode_detectors(IDetector.TYPE_BUSLANE, 'bus-lane.conf')
    parking_confs = decode_detectors(IDetector.TYPE_PARKING, 'parking.conf')
    redlight_confs = decode_detectors(IDetector.TYPE_REDLIGHT, 'red-light.conf')
    redlight_le_confs = decode_detectors(IDetector.TYPE_REDLIGHT, 'railroad-crossing.conf')
    sectionspeed_confs = decode_detectors(IDetector.TYPE_SECTIONSPEED, 'section-speed.conf')
    spotspeed_confs = decode_detectors(IDetector.TYPE_SPOTSPEED, 'spot-speed.conf')
    spotspeed_waga_confs = decode_detectors(IDetector.TYPE_SPOTSPEED_WAGA, 'spot-speed-waga.conf')
    wanted_confs = decode_detectors(IDetector.TYPE_WANTED, 'wanted.conf')
    wrongway_confs = decode_detectors(IDetector.TYPE_WRONG_WAY, 'wrong-way.conf')
    seatbelts_confs = decode_detectors(IDetector.TYPE_SEATBELT, 'seatbelt.conf')

    return DetectorsConf(
        base_confs,
        transit_confs,
        parking_confs,
        wrongway_confs,
        spotspeed_confs,
        spotspeed_waga_confs,
        redlight_confs,
        wanted_confs,
        sectionspeed_confs,
        redlight_le_confs,
        buslane_confs,
        seatbelts_confs)
