from typing import Dict, Optional

from edge import eventlog
from edge.image.types.jnode.builder import decode__im_builder_conf
from edge.types.exceptions import JnodeError
from edge.types.identity import Identity

from speedcontrol.detectors.redlight_leddar.types.conf import DetectorConf, RuleConf
from speedcontrol.detectors.types.jnode.conf import decode__base_rule_conf, decode__video_conf


def decode__rule_conf(rule_jnode: Dict, footer_defvars: Dict) -> Optional[RuleConf]:
    """
    :raise JnodeError:
    """
    if type(rule_jnode) != dict:
        eventlog.log_e_param('#/')
        raise JnodeError

    # deserialize #/name
    # deserialize #/enabled
    # deserialize #/ticket
    rule = decode__base_rule_conf(rule_jnode)
    if rule.enabled:
        return RuleConf(rule.name, rule.ticket, footer_defvars)


def decode__detector_conf(identity: Identity, jnode: Dict, footer_defvars: Dict) -> DetectorConf:
    """
    :raise JnodeError:
    """
    # deserialize #/name
    name = jnode.get('name')
    if not type(name) in [str] or len(name) > 32:
        eventlog.log_e_param('#/name')
        raise JnodeError

    # deserialize #/store
    store = jnode.get('store')
    if not (type(store) in [str] and 1 <= len(store) <= 2):
        eventlog.log_e_param('#/store')
        raise JnodeError

    # deserialize #/rules
    rules_jnode = jnode.get('rules')
    if type(rules_jnode) != list:
        eventlog.log_e_param('#/rules')
        raise JnodeError

    rules = []
    for rule_jnode in rules_jnode:
        rule = decode__rule_conf(rule_jnode, footer_defvars)
        if rule:
            rules.append(rule)

    # deserialize #/images
    images_jnode = jnode.get('images')
    if type(images_jnode) != dict:
        eventlog.log_e_param('#/images')
        raise JnodeError

    im_engine = decode__im_builder_conf(images_jnode['violation'])

    # deserialize #/videos/measuments
    video_jnode = jnode.get('videos', {}).get('measurement')
    video_args = decode__video_conf(video_jnode) if video_jnode else None
    return DetectorConf(identity, name, store, rules, im_engine, video_args)
