from typing import Dict, Final, List, Tuple

from edge.image.types.builder import EventMeasImBuilderConf
from edge.types.dir import Dir

from edge.support.schedule import ScheduleConf
from speedcontrol.events.ticket import TicketInfo
from edge.support.mask import MaskRegion
from edge.types.identity import Identity


class RuleLimits:
    def __init__(self, speed_limit: int, over_speed: int):
        self.speed_limit: Final[int] = speed_limit
        self.over_speed: Final[int] = over_speed


class RuleConf:
    def __init__(self,
                 name: str,
                 enabled: bool,
                 schedule: ScheduleConf,
                 ticket: TicketInfo,
                 limits: RuleLimits,
                 footer_defvars: Dict):

        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits
        self.footer_defvars = footer_defvars


class InitAreaConf:
    def __init__(self, controller: str, auth: Tuple[str, str], origin: str):
        self.controller = controller
        self.auth = auth
        self.origin = origin


class FiniAreaConf:
    def __init__(self, mask: MaskRegion, scene_wh: Tuple[int, int]):
        self.mask = mask
        self.scene_wh = scene_wh


class SectionConf:
    def __init__(self,
                 init_area: InitAreaConf,
                 fini_area: FiniAreaConf,
                 direction: Dir,
                 dist: float,
                 asssert_dist: float = 1000):

        self.init_area = init_area
        self.fini_area = fini_area
        self.direction = direction
        self.dist = dist
        self.asssert_dist = asssert_dist


class DetectorConf:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 rules: List[RuleConf],
                 im_engine: EventMeasImBuilderConf,
                 section: SectionConf,
                 ignore_ra: bool = False):

        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.im_engine = im_engine
        self.section = section
        self.ignore_ra = ignore_ra
