from typing import Dict

from edge.types.identity import Identity
from speedcontrol.detectors.spotspeed.types.stats import DetectorStats, RuleStats


def encode__rule_stats(name: str, stats: RuleStats) -> Dict:
    make_evidence_total_fail = stats.evidence_failed_make_images + stats.evidence_failed_load_images
    total_fail = (stats.schedule_unmatch +
                  stats.lane_unmatch +
                  stats.raspeed_valid +
                  stats.class_unmatch)

    return {
        'type': 'spot-speed~rule',
        'name': name,
        'stat': {
            'assert-track': {
                'count': [
                    stats.total,
                    stats.total - total_fail,
                    total_fail
                ],
                'type-unmatch': stats.class_unmatch,
                'schedule-unmatch': stats.schedule_unmatch,
                'lane-unmatch': stats.lane_unmatch,
                'ra-speed-valid': stats.raspeed_valid
            },
            'make-evidence': {
                'count': [
                    stats.evidence_exec,
                    stats.evidence_exec - make_evidence_total_fail,
                    make_evidence_total_fail
                ],
                'load-images': stats.evidence_failed_load_images,
                'make-images': stats.evidence_failed_make_images
            }
        }
    }


def encode__detector_stats(identity: Identity, stats: DetectorStats) -> Dict:
    total_fails = stats.vrp_absent + stats.vrp_text_invalid + stats.ra_track_absent + stats.main_speed_absent
    return {
        'type': 'spot-speed',
        'name': f'{identity.ref}',
        'stat': {
            'assert-track': {
                'count': [stats.total, stats.total - total_fails, total_fails],
                'ra-absent': stats.ra_track_absent,
                'lr-absent': stats.vrp_absent,
                'lr-invalid': stats.vrp_text_invalid,
                'main-speed-absent': stats.main_speed_absent,
            },
            'emit-event': {
                'count': [
                    stats.emit_total,
                    stats.emit_total - stats.emit_fail,
                    stats.emit_fail
                ]
            }
        }
    }
