from typing import Optional, Dict

from edge.services.types.tz import ITzService
from edge.types.dir import Dir
from edge.types.jnode.dir import encode_dir_text
from edge.types.origin import Origin

from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence
from speedcontrol.events.jnode.schedule import encode__schedule_info


class Evidence(BaseEvidence):
    def __init__(self,
                 origin: Origin,
                 ts: int,
                 section_speed: float,
                 section_overspeed: float,
                 detector_info: DetectorInfo,
                 direction: Dir,
                 detector_dist: float,
                 measured_dist: float,
                 control_ts: Optional[int] = None):

        super().__init__(BaseEvidence.EVENT_TYPE__SECTIONSPEED, origin, ts, control_ts=control_ts)
        self.detector = detector_info
        self.section_speed = section_speed
        self.section_overspeed = section_overspeed
        self.dir = direction
        self.detector_dist = detector_dist
        self.measured_dist = measured_dist

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)
        waypoints_jnode = list()
        for waypoint in self._waypoints:
            waypoints_jnode.append(waypoint.serialize(tz_service))

        jnode['evidence'] = {
            'detector': {
                'name': self.detector.name,
                'dir': encode_dir_text(self.dir),
                'dist': self.detector_dist,
                'rule': {
                    'name': self.detector.rule.name,
                    'schedule': encode__schedule_info(self.detector.rule.schedule),
                    'ticket': {
                        'name': self.detector.rule.ticket.name
                    },
                    'limits': self.detector.rule.limits
                }
            },
            'measurements': {
                'section-dist': self.measured_dist,
                'section-speed': self.section_speed,
                'section-overspeed': self.section_overspeed,
                'waypoints': {
                    'init': waypoints_jnode[0],
                    'fini': waypoints_jnode[1]
                }
            }
        }

        return jnode
