from typing import Dict

from edge.services.types.tz import ITzService
from edge.types.origin import Origin

from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence
from speedcontrol.events.jnode.schedule import encode__schedule_info


class Evidence(BaseEvidence):
    def __init__(self,
                 origin: Origin,
                 ts: int,
                 duration: int,
                 detector_info: DetectorInfo):

        super().__init__(BaseEvidence.EVENT_TYPE__WRONGWAY, origin, ts)
        self.detector = detector_info
        self.duration = duration

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)
        jnode['evidence'] = {

        }
        waypoints_jnode = list()
        for waypoint in self._waypoints:
            waypoints_jnode.append(waypoint.serialize(tz_service))

        jnode['evidence'] = {
            'detector': {
                'name': self.detector.name,
                'rule': {
                    'name': self.detector.rule.name,
                    'schedule': encode__schedule_info(self.detector.rule.schedule),
                    'ticket': {
                        'name': self.detector.rule.ticket.name
                    },
                    'limits': self.detector.rule.limits
                }
            },
            'measurements': {
                'duration': self.duration,
                'waypoints': {
                    'violation': waypoints_jnode
                }
            }
        }

        if len(self._videos):
            videos_jnode = [x.serialize(tz_service) for x in self._videos]
            jnode['evidence']['measurements']['videos'] = videos_jnode

        return jnode
