from typing import Dict, Final, List, Optional

from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.location import Location

from speedcontrol.events.jnode.vehicle import encode__violation_vehicle
from speedcontrol.events.vehicle import ViolationVehicle
from speedcontrol.events.waypoint import Waypoint


class ViolationWaypoint(Waypoint):
    def __init__(self,
                 name: str,
                 main_ts: int,
                 control_ts: Optional[int],
                 location: Optional[Location],
                 images: List[MemBlob] | None,
                 vehicle: Optional[ViolationVehicle]):

        super().__init__(name, main_ts, control_ts, location, images)

        self.vehicle: Final[ViolationVehicle] = vehicle

    def serialize(self, tz_service: ITzService, with_images: bool = True) -> Dict:
        jnode = Waypoint.serialize(self, tz_service, with_images)

        vehicle_jnode: Optional[Dict] = encode__violation_vehicle(self.vehicle)
        if vehicle_jnode is not None:
            jnode.update({
                'vehicle': vehicle_jnode,
            })

        return jnode
