from typing import Callable, Dict, Set, List

from edge import eventlog
from edge.services.react import ReactSource
from edge.types.jnode.trackset import decode__tracksets_le
from edge.types.trackset import Trackset

from speedcontrol.services.types.leddar import ILeddarService
from speedcontrol.services.types.react_le import ILeReactService


class _ReactService(ILeReactService):
    def __init__(self, twisted_reactor, le_service: ILeddarService):
        super().__init__()
        self._le_service = le_service
        self._source = ReactSource(twisted_reactor)
        self._source.set_handler(self._handle_le_react)
        self._handler: Callable[[Trackset], None] | None = None

    def destroy(self) -> None:
        self._source.destroy()

    def set_handler(self, handler: Callable[[str, Dict], None]) -> None:
        self._source.set_handler(handler)

    def set_endpoints(self, endpoints: Set[str]) -> None:
        self._source.set_endpoints(endpoints)

    def _handle_le_react(self, react_name: str, react_jnode: Dict) -> None:
        if self._handler is None:
            return

        try:
            tracksets: List[Trackset] = decode__tracksets_le(react_jnode)
        except (KeyError, ValueError) as x:
            eventlog.log_e(f'fail: unpack {react_name} <{x.message}>')
            return

        if len(tracksets):
            self._le_service.update(react_jnode)

        for trackset in tracksets:
            self._handler(trackset)


def init_service(twisted_reactor, endpoints: Set[str], le_service: ILeddarService) -> ILeReactService:
    service = _ReactService(twisted_reactor, le_service)
    service.set_endpoints(endpoints)
    return service
