from typing import Dict

from edge import eventlog
from edge.services.types.jnode.lane import decode__lanes_conf
from edge.services.types.lane import LaneServiceConf
from edge.types.exceptions import JnodeError

from speedcontrol.services.types.road import RoadConf


def decode__road_conf(jnode: Dict) -> RoadConf:
    """
    :param jnode:
        /etc/edge/speedcontrol.10/road.conf#/
    :raise JnodeError:
    """
    if not len(jnode):
        eventlog.log_i('using stub LaneDetector')
        stub_lane_service = LaneServiceConf([], 0, 0)
        return RoadConf(stub_lane_service)

    # decode #/road
    road_jnode = jnode.get('road')
    if not type(road_jnode) in [dict]:
        eventlog.log_e_param('#/road')
        raise JnodeError

    # decode #/road/dir
    dir_jnode = road_jnode.get('dir')
    if not type(road_jnode) in [dict]:
        eventlog.log_e_param('#/road/dir')
        raise JnodeError

    # decode #/road/dir/approaching
    dir_alias_a = dir_jnode.get('approaching')
    if not type(dir_alias_a) in [str]:
        eventlog.log_e_param('#/road/dir/approaching')
        raise JnodeError

    # decode #/road/dir/receding
    dir_alias_r = dir_jnode.get('receding')
    if not type(dir_alias_r) in [str]:
        eventlog.log_e_param('#/road/dir/receding')
        raise JnodeError

    # decode #/road/dir/unknown
    dir_alias_u = dir_jnode.get('unknown')
    if not type(dir_alias_u) in [str]:
        eventlog.log_e_param('#/road/dir/unknown')
        raise JnodeError

    # decode #/scene-wh
    wh_jnode = jnode.get('scene-wh')
    if not wh_jnode:
        eventlog.log_e_param('#/scene-wh')
        raise JnodeError
    elif len(wh_jnode) != 2:
        eventlog.log_e_param('#/scene-wh')
        raise JnodeError

    mask_w, mask_h = wh_jnode
    if not (type(mask_w) in [int] and 0 <= mask_w <= 65535):
        eventlog.log_e_param('#/mask/scene-wh/0')
        raise JnodeError

    elif not (type(mask_h) in [int] and 0 <= mask_h <= 65535):
        eventlog.log_e_param('#/mask/scene-wh/1')
        raise JnodeError

    # decode #/road/lanes
    lanes_jnode = road_jnode.get('lanes', [])
    lane_service_conf = decode__lanes_conf(lanes_jnode, mask_w, mask_h)

    return RoadConf(lane_service_conf, dir_alias_a, dir_alias_r, dir_alias_u)
