import ctypes
from typing import Callable, Optional

from edge.services.types.service import IService


class SignServiceConf:
    def __init__(self, iit_conf_path: str, ui_conf_path: str, ui_ref: str, log_level: str = 'info'):
        self.iit_conf_path = iit_conf_path
        self.ui_conf_path = ui_conf_path
        self.ui_ref = ui_ref
        self.log_level = log_level


class SignError(RuntimeError):
    pass


class ISignService(IService):
    def __init__(self):
        super().__init__()

    @property
    def cert(self) -> bytes:
        raise NotImplementedError

    @property
    def hash_type(self) -> str:
        raise NotImplementedError

    def hash_blob(self, blob_buffer: bytes, hash_buffer: ctypes.Array) -> int:
        """
        :raise SignError:
        """
        raise NotImplementedError

    def sign_blob(self, blob_buffer: bytes, sign_handler: Callable[[Optional[str], Optional[str]], None]) -> None:
        """
        :raise SignError:
        """
        raise NotImplementedError
