from os.path import isabs

from edge import eventlog
from tracker.services.types.lr_track_provider import LrTrackProviderConf, LrTrackImageQueueConf, LrTrackAccConf, LrTrackProviderStats, \
    LrTrackFilterRuleConf, LrTrackFilterConf
from tracker.types.lr import LrTrackStat


def decode__lr_track_image_queue_conf(jnode: dict) -> LrTrackImageQueueConf:
    """
    :param jnode:
        main.conf#/services/track-provider/lost-acc/store
    """
    endpoint = jnode.get('endpoint')
    if not (str == type(endpoint) and len(endpoint) < 64 and isabs(endpoint)):
        raise ValueError('#/endpoint')

    max_file_count = jnode.get('max-file-count', 15)
    if not (int == type(max_file_count) and 15 <= max_file_count <= 50):
        raise ValueError('#/max-file-count')

    return LrTrackImageQueueConf(endpoint, max_file_count - 5, max_file_count)


def decode__lr_track_filter_conf(jnode: list[str]) -> LrTrackFilterConf:
    """
    :param jnode:
        main.conf#/services/track-provider/filter
    """
    filter_rules: list[LrTrackFilterRuleConf] = []
    if len(jnode) > 0:
        filter_rules.append(LrTrackFilterRuleConf(vrp_templates=set(jnode)))

    return LrTrackFilterConf(filter_rules)


def decode__lr_track_acc_conf(jnode: dict) -> LrTrackAccConf:
    imq = decode__lr_track_image_queue_conf(jnode['store'])

    tout_sec = jnode['timeout']
    if not (0 <= tout_sec <= 30):
        raise ValueError('main.conf#/fact-event/debounce-tout')

    return LrTrackAccConf(imq, tout_sec)


def decode__lr_track_provider_mode(values: list[str]) -> set[LrTrackStat]:
    mode: set[LrTrackStat] = set()

    for value in values:
        if value == 'detected':
            mode.add(LrTrackStat.DETECTED)
        elif value == 'lost':
            mode.add(LrTrackStat.LOST)
        else:
            eventlog.log_e(f'fail: unsupported (mode:{value})')
            raise NotImplementedError

    return mode


def decode__lr_track_provider_conf(jnode: dict) -> LrTrackProviderConf:
    """
    :param jnode:
        main.conf#/services/track-provider
    """
    react_dir = jnode['endpoint']
    mode: set[LrTrackStat] = decode__lr_track_provider_mode(jnode['mode'])
    track_filter = decode__lr_track_filter_conf(jnode['filter'] or [])
    lost_track_acc = decode__lr_track_acc_conf(jnode['lost-track-acc'])

    return LrTrackProviderConf(react_dir, mode, track_filter, lost_track_acc)


def encode__lr_track_provider_stats(stats: LrTrackProviderStats) -> dict:
    consume_mode: list[str] = []

    if LrTrackStat.DETECTED in stats.mode:
        consume_mode.append('detected')

    if LrTrackStat.LOST in stats.mode:
        consume_mode.append('lost')

    return {
        'type': 'tracker~lr-track-provider',
        'name': 'tracker.0~lr-track-provider.0',
        'stat': {
            'consume': {
                'count': {
                    'filtered': stats.consume.filtered_count,
                    'detected': stats.consume.detected_count,
                    'lost': stats.consume.lost_count
                },
                'mode': consume_mode
            },
            'accumulate-lost': {
                'count': {
                    'create': stats.acc_lost.success_create_count,
                    'append': stats.acc_lost.success_append_count
                },
                'errors': {
                    'copy-image': stats.acc_lost.failure_copy_image
                }
            },
            'produce': {
                'count': {
                    'detected': stats.produce.detected_count,
                    'lost': stats.produce.lost_count
                }
            }
        }
    }
