from typing import Dict

from tracker.services.types.timekeeper import TimekeeperConf, ChangelogConf, ChronycConf


def decode__timekeeper_conf(jnode: Dict) -> TimekeeperConf:
    #
    # {
    #     "changelog": {
    #         "max-depth-spans": 10,
    #         "max-depth-sec": 600,
    #         "span-tout-sec": 2.0
    #     },
    #     "chronyc": {
    #         "interval-sec": 1.0,
    #         "max-correction-sec": 0.03,
    #         "max-skew-ppm": 0.350
    #     }
    # }

    changelog_jnode = jnode.get('changelog')
    if not type(changelog_jnode) in [dict]:
        raise ValueError('main.conf#/Timekeeper/Changelog')

    raw_max_depth_spans = changelog_jnode.get('max-depth-spans')
    if (not type(raw_max_depth_spans) in [int]) and 0 <= raw_max_depth_spans <= 255:
        raise ValueError('main.conf#/Timekeeper/Changelog/MaxDepthSpans')

    raw_max_depth_sec = changelog_jnode.get('max-depth-sec')
    if (not type(raw_max_depth_sec) in [int]) and 0 <= raw_max_depth_sec <= 65535:
        raise ValueError('main.conf#/Timekeeper/Changelog/MaxDepthSec')

    raw_span_tout_sec = changelog_jnode.get('span-tout-sec')
    if (not type(raw_span_tout_sec) in [int, float]) and 0 <= raw_span_tout_sec <= 255:
        raise ValueError('main.conf#/Timekeeper/Changelog/SpanToutSec')

    chronyc_jnode = jnode.get('chronyc')
    if not type(chronyc_jnode) in [dict]:
        raise ValueError('main.conf#/Timekeeper/Chronyc')

    raw_interval_sec = chronyc_jnode.get('interval-sec')
    if (not type(raw_interval_sec) in [int, float]) and 0 <= raw_interval_sec <= 255:
        raise ValueError('main.conf#/Timekeeper/Chronyc/IntervalSec')

    raw_max_correction_sec = chronyc_jnode.get('max-correction-sec')
    if (not type(raw_max_correction_sec) in [int, float]) and 0 <= raw_max_correction_sec <= 255:
        raise ValueError('main.conf#/Timekeeper/Chronyc/MaxCorrectionSec')

    raw_max_skew_ppm = chronyc_jnode.get('max-skew-ppm', 0)
    if (not type(raw_max_skew_ppm) in [int, float]) and 0 <= raw_max_skew_ppm <= 255:
        raise ValueError('main.conf#/Timekeeper/Chronyc/MaxSkewPpm')

    changelog = ChangelogConf(raw_max_depth_spans, raw_max_depth_sec, raw_span_tout_sec)
    chronyc = ChronycConf(1, raw_interval_sec, raw_max_correction_sec, raw_max_skew_ppm)
    return TimekeeperConf(changelog, chronyc)


