from abc import abstractmethod, ABCMeta
from typing import Final

from edge.services.types.service import IService
from tracker.types.lr import LrTrackStat, LrTrackSet


class LrTrackConsumeStats:
    def __init__(self) -> None:
        self.filtered_count: int = 0
        self.detected_count: int = 0
        self.lost_count: int = 0


class LrTrackLostAccStats:
    def __init__(self) -> None:
        self.success_create_count: int = 0
        self.success_append_count: int = 0
        self.failure_copy_image: int = 0


class LrTrackProduceStats:
    def __init__(self) -> None:
        self.detected_count: int = 0
        self.lost_count: int = 0


class LrTrackProviderStats:
    def __init__(self, mode: set[LrTrackStat]) -> None:
        self.consume = LrTrackConsumeStats()
        self.acc_lost = LrTrackLostAccStats()
        self.produce = LrTrackProduceStats()
        self.mode: Final[set[LrTrackStat]] = mode


class LrTrackFilterRuleConf:
    def __init__(self, vrp_templates: set[str]) -> None:
        """
        :param vrp_templates:
            templates of plates to be ignored
        """
        self.vrp_templates: set[str] = vrp_templates


class LrTrackFilterConf:
    def __init__(self, rules: list[LrTrackFilterRuleConf]) -> None:
        """
        :param rules:
            list of active rules
        """
        self.rules = rules


class LrTrackImageQueueConf:
    def __init__(self, endpoint: str, min_file_count: int, max_file_count: int) -> None:
        self.endpoint: Final[str] = endpoint
        self.min_file_count: Final[int] = min_file_count
        self.max_file_count: Final[int] = max_file_count
        self.log_level = 'info'


class LrTrackAccConf:
    def __init__(self, imq: LrTrackImageQueueConf, tout_sec: int) -> None:
        self.imq: Final[LrTrackImageQueueConf] = imq
        self.tout_sec: Final[int] = tout_sec


class LrTrackProviderConf:
    def __init__(self, react_dir: str, mode: set[LrTrackStat],
                 track_filter: LrTrackFilterConf,
                 lost_track_acc: LrTrackAccConf) -> None:

        self.mode: Final[set[LrTrackStat]] = mode
        self.react_dir: Final[str] = react_dir
        self.track_filter: Final[LrTrackFilterConf] = track_filter
        self.lost_track_acc: Final[LrTrackAccConf] = lost_track_acc


class ILrTrackProviderHandler(metaclass=ABCMeta):
    @abstractmethod
    def handle_detected_lr_track(self, lr_trackset: LrTrackSet) -> None:
        pass

    @abstractmethod
    def handle_lost_lr_track(self, lr_trackset: LrTrackSet) -> None:
        pass


class ILrTrackProvider(IService):
    pass
