from abc import ABCMeta, abstractmethod
from typing import Final

from edge.services.types.service import IService
from edge.types.trend import LrTrend
from tracker.types.ra import RaTrack, RaSceneItem


class RaTrackAddStats:
    def __init__(self) -> None:
        self.count = 0


class RaTrackPopStats:
    def __init__(self) -> None:
        self.success_count = 0
        self.failure_not_found_count = 0
        self.failure_too_short_count = 0
        self.failure_mmr_speed_limit_count = 0


class RaTrackDelStats:
    def __init__(self) -> None:
        self.success_count = 0
        self.failure_too_short_count = 0


class RaTrackProviderStats:
    def __init__(self) -> None:
        self.add_track = RaTrackAddStats()
        self.pop_track = RaTrackPopStats()
        self.del_track = RaTrackDelStats()


class RaTrackFilterRuleConf:
    def __init__(self, mmr_classes: set[str], max_speed: int) -> None:
        """
        :param mmr_classes:
            MMR classes
        :param max_speed:
            Max speed, km/h
        """
        self.mmr_classes: set[str] = mmr_classes
        self.max_speed: int = max_speed


class RaTrackFilterConf:
    def __init__(self, rules: list[RaTrackFilterRuleConf]) -> None:
        """
        :param rules:
            list of active rules
        """
        self.rules = rules


class RaTrackProviderConf:
    def __init__(self, require_plate: bool, ratrack_filter: RaTrackFilterConf, tout_sec: int = 15, min_track_len: int = 5) -> None:
        self.require_plate: bool = require_plate
        self.ratrack_filter: RaTrackFilterConf = ratrack_filter
        self.tout_sec: Final[int] = tout_sec
        self.min_track_len: Final[int] = min_track_len


class IRaTrackProviderHandler(metaclass=ABCMeta):
    @abstractmethod
    def handle_lost_ra_track(self, ra_track: RaTrack) -> None:
        pass


class IRaTrackProvider(IService):
    @abstractmethod
    def add_scene(self, items: list[RaSceneItem]) -> None:
        pass

    @abstractmethod
    def pop_track(self, ra_oids_desc: list[int], mmr_class: str, lr_trend: LrTrend) -> RaTrack | None:
        pass
