from typing import Final, Self

from edge.types.shape.cuboid import CuboidI
from tracker.types.lr import LrSceneItem, LrTrackSet
from tracker.types.ra import RaSceneItem, RaTrack


class LrRaSceneItem:
    def __init__(self,
                 lr_item: LrSceneItem | None,
                 ra_item: RaSceneItem,
                 ra_cuboid: CuboidI) -> None:

        if not lr_item and not ra_item:
            raise ValueError

        self.lr_item: Final[LrSceneItem | None] = lr_item
        self.ra_item: Final[RaSceneItem] = ra_item
        self.ra_cuboid: Final[CuboidI] = ra_cuboid


class LrRaScene:
    def __init__(self,
                 lrra_items: list[LrRaSceneItem],
                 lr_items: list[LrSceneItem],
                 ra_items: list[LrRaSceneItem]) -> None:
        """
        :param lrra_items:
            The list of matched lr_scene_item and ra_scene_item
        :param lr_items:
            The list of not matched lr_scene_items
        :param ra_items:
            The list of not matched ra_scene_items
        """
        self.lrra_items: Final[list[LrRaSceneItem]] = lrra_items
        self.lr_items: Final[list[LrSceneItem]] = lr_items
        self.ra_items: Final[list[LrRaSceneItem]] = ra_items

    @classmethod
    def create(cls, tr_items: list[LrRaSceneItem], lr_items: list[LrSceneItem], ra_items: list[LrRaSceneItem]):
        return cls(tr_items, lr_items, ra_items)

    @classmethod
    def create_lr(cls, lr_items: list[LrSceneItem]) -> Self:
        return cls([], lr_items, [])

    @classmethod
    def create_ra(cls, ra_items: list[LrRaSceneItem]) -> Self:
        return cls([], [], ra_items)

    @classmethod
    def create_empty(cls) -> Self:
        return cls([], [], [])


class LrRaTrackset:
    def __init__(self, lr_trackset: LrTrackSet | None, ra_track: RaTrack | None) -> None:
        if not lr_trackset and not ra_track:
            raise ValueError

        self.lr_trackset: Final[LrTrackSet | None] = lr_trackset
        self.ra_track: Final[RaTrack | None] = ra_track
